/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "background.h"
#include "gamebody.h"
#include "SDL.h"
#include "bodyfactory.h"
#include "debugmsg.h"

Background::Background() {}

void Background::draw(real zoom,real theta,const Vector2d & eye) {
    theta=-theta;
    real f=zoom/(zoom*7+1);
    real f8=f*8;
    Matrix2x3 view(f,f,theta,-eye.x,-eye.y,Graphic::screencenter.x,Graphic::screencenter.y);

    theta=-theta;

    for (std::list<GameBody*>::iterator i=objects.begin();i!=objects.end();i++)
        (*i)->draw(view*(*i)->getPosition(),f8,theta);
}

#define transf(px,py) (pos+(Vector2d(px,py)-Vector2d(100,100))*zoom)

std::list<GameBody*> starlist(std::string group,int x,int y,real zoom) {
    std::list<GameBody*> res;
    Vector2d pos(x,y);
    if (group=="farstargroup1") {
        res.push_front(BodyFactory::instance()->create("farstar1",transf(72,79)));
        res.push_front(BodyFactory::instance()->create("farstar1",transf(60,166)));
        res.push_front(BodyFactory::instance()->create("farstar2",transf(44,38)));
        res.push_front(BodyFactory::instance()->create("farstar4",transf(69,12)));
        res.push_front(BodyFactory::instance()->create("farstar4",transf(92,42)));
        res.push_front(BodyFactory::instance()->create("farstar4",transf(106,60)));
        res.push_front(BodyFactory::instance()->create("farstar4",transf(58,133)));
        res.push_front(BodyFactory::instance()->create("farstar4",transf(29,121)));
        res.push_front(BodyFactory::instance()->create("farstar4",transf(48,47)));
        res.push_front(BodyFactory::instance()->create("farstar4",transf(137,168)));
        res.push_front(BodyFactory::instance()->create("farstar4",transf(93,125)));
        res.push_front(BodyFactory::instance()->create("farstar1",transf(112,78)));
        res.push_front(BodyFactory::instance()->create("farstar2",transf(91,95)));
        res.push_front(BodyFactory::instance()->create("farstar5",transf(187,34)));
        res.push_front(BodyFactory::instance()->create("farstar3",transf(76,80)));
        res.push_front(BodyFactory::instance()->create("farstar1",transf(125,101)));
        res.push_front(BodyFactory::instance()->create("farstar7",transf(117,131)));
        res.push_front(BodyFactory::instance()->create("farstar7",transf(25,83)));
        res.push_front(BodyFactory::instance()->create("farstar7",transf(18,45)));
        res.push_front(BodyFactory::instance()->create("farstar7",transf(53,152)));
        res.push_front(BodyFactory::instance()->create("farstar4",transf(52,74)));
        res.push_front(BodyFactory::instance()->create("farstar7",transf(155,127)));
        res.push_front(BodyFactory::instance()->create("farstar8",transf(123,43)));
        res.push_front(BodyFactory::instance()->create("farstar8",transf(156,63)));
        res.push_front(BodyFactory::instance()->create("farstar9",transf(159,136)));
    } else if (group=="farstargroup2") {
        res.push_front(BodyFactory::instance()->create("farstar10",transf(25,46)));
        res.push_front(BodyFactory::instance()->create("farstar10",transf(43,64)));
        res.push_front(BodyFactory::instance()->create("farstar10",transf(66,39)));
        res.push_front(BodyFactory::instance()->create("farstar10",transf(83,7)));
        res.push_front(BodyFactory::instance()->create("farstar10",transf(163,31)));
        res.push_front(BodyFactory::instance()->create("farstar10",transf(121,64)));
        res.push_front(BodyFactory::instance()->create("farstar10",transf(159,65)));
        res.push_front(BodyFactory::instance()->create("farstar10",transf(160,76)));
        res.push_front(BodyFactory::instance()->create("farstar10",transf(174,87)));
        res.push_front(BodyFactory::instance()->create("farstar10",transf(161,96)));
        res.push_front(BodyFactory::instance()->create("farstar10",transf(116,100)));
        res.push_front(BodyFactory::instance()->create("farstar10",transf(175,167)));
        res.push_front(BodyFactory::instance()->create("farstar10",transf(95,169)));
        res.push_front(BodyFactory::instance()->create("farstar10",transf(28,136)));
        res.push_front(BodyFactory::instance()->create("farstar10",transf(97,134)));
        res.push_front(BodyFactory::instance()->create("farstar1",transf(26,80)));
        res.push_front(BodyFactory::instance()->create("farstar11",transf(59,156)));
        res.push_front(BodyFactory::instance()->create("farstar11",transf(48,104)));
        res.push_front(BodyFactory::instance()->create("farstar11",transf(183,108)));
        res.push_front(BodyFactory::instance()->create("farstar10",transf(124,78)));
        res.push_front(BodyFactory::instance()->create("farstar12",transf(69,149)));
        res.push_front(BodyFactory::instance()->create("farstar1",transf(113,140)));
        res.push_front(BodyFactory::instance()->create("farstar1",transf(71,112)));
        res.push_front(BodyFactory::instance()->create("farstar1",transf(148,111)));
        res.push_front(BodyFactory::instance()->create("farstar7",transf(98,48)));
        res.push_front(BodyFactory::instance()->create("farstar7",transf(125,75)));
        res.push_front(BodyFactory::instance()->create("farstar7",transf(98,99)));
        res.push_front(BodyFactory::instance()->create("farstar4",transf(66,159)));
        res.push_front(BodyFactory::instance()->create("farstar4",transf(91,99)));
        res.push_front(BodyFactory::instance()->create("farstar10",transf(47,170)));
        res.push_front(BodyFactory::instance()->create("farstar13",transf(147,134)));
    } else if (group=="farstargroup3") {
        res.push_front(BodyFactory::instance()->create("farstar2",transf(103,174)));
        res.push_front(BodyFactory::instance()->create("farstar2",transf(33,80)));
        res.push_front(BodyFactory::instance()->create("farstar5",transf(170,130)));
        res.push_front(BodyFactory::instance()->create("farstar14",transf(29,118)));
        res.push_front(BodyFactory::instance()->create("farstar14",transf(141,24)));
    } else {
        CHERROR<<"starlist: group "<<group<<" unknown."<<ENDL;
    }
    return res;
}
