/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "clipping.h"

bool clipSegment(int w,int h,Vector2d * v1,Vector2d * v2) {
    if (point_inside(w,h,v1->x,v1->y)) {
        if (point_inside(w,h,v2->x,v2->y)) {
            return true; // Both inside
        } else { // v1 inside, v2 outside
            if (v2->x<0) {
                v2->y+=(v2->y-v1->y)*v2->x/(v1->x-v2->x);
                v2->x=0;
            } else if (v2->x>=w) {
                v2->y+=(v1->y-v2->y)*(v2->x-(w-1))/(v2->x-v1->x);
                v2->x=w-1;
            }

            if (v2->y<0) {
                v2->x+=(v2->x-v1->x)*v2->y/(v1->y-v2->y);
                v2->y=0;
            } else if (v2->y>=h) {
                v2->x+=(v1->x-v2->x)*(v2->y-(h-1))/(v2->y-v1->y);
                v2->y=h-1;
            }
        }
    } else if (point_inside(w,h,v2->x,v2->y)) {
        // v1 outside, v2 inside
        if (v1->x<0) {
            v1->y+=(v1->y-v2->y)*v1->x/(v2->x-v1->x);
            v1->x=0;
        } else if (v1->x>=w) {
            v1->y+=(v2->y-v1->y)*(v1->x-(w-1))/(v1->x-v2->x);
            v1->x=w-1;
        }

        if (v1->y<0) {
            v1->x+=(v1->x-v2->x)*v1->y/(v2->y-v1->y);
            v1->y=0;
        } else if (v1->y>=h) {
            v1->x+=(v2->x-v1->x)*(v1->y-(h-1))/(v1->y-v2->y);
            v1->y=h-1;
        }
    } else { // Both outside

        if ((v1->x<0 && v2->x<0) || (v1->x>=w && v2->x>=w) ||
                (v1->y<0 && v2->y<0) || (v1->y>=h && v2->y>=h))  return false;

        if (v2->x<0) {
            v2->y+=(v2->y-v1->y)*v2->x/(v1->x-v2->x);
            v2->x=0;
        } else if (v2->x>=w) {
            v2->y+=(v1->y-v2->y)*(v2->x-(w-1))/(v2->x-v1->x);
            v2->x=w-1;
        }

        if (v2->y<0) {
            v2->x+=(v2->x-v1->x)*v2->y/(v1->y-v2->y);
            v2->y=0;
        } else if (v2->y>=h) {
            v2->x+=(v1->x-v2->x)*(v2->y-(h-1))/(v2->y-v1->y);
            v2->y=h-1;
        }

        if (v1->x<0) {
            v1->y+=(v1->y-v2->y)*v1->x/(v2->x-v1->x);
            v1->x=0;
        } else if (v1->x>=w) {
            v1->y+=(v2->y-v1->y)*(v1->x-(w-1))/(v1->x-v2->x);
            v1->x=w-1;
        }

        if (v1->y<0) {
            v1->x+=(v1->x-v2->x)*v1->y/(v2->y-v1->y);
            v1->y=0;
        } else if (v1->y>=h) {
            v1->x+=(v2->x-v1->x)*(v1->y-(h-1))/(v1->y-v2->y);
            v1->y=h-1;
        }

        return (0<=v1->x && v1->x<w && 0<=v1->y && v1->y<h) && (0<=v2->x && v2->x<w && 0<=v2->y && v2->y<h);
    }
    return true;
}

inline bool intersectsRect(int x11,int y11,int x12,int y12,
                           int x21,int y21,int x22,int y22) {
    if (x21>=x12) return false;
    if (y21>=y12) return false;
    if (x11>=x22) return false;
    if (y11>=y22) return false;

    return true;
}

/**
 * @returns true if some part of the rectangle is visible
 */
bool clipRectangle(SDL_Rect*r,int x,int y,int w,int h) {
    if (!intersectsRect(r->x,r->y,r->x+r->w,r->y+r->h,x,y,x+w,y+h))
        return false;

    if (r->x<x) {
        r->w-=x-r->x;
        r->x=x;
    }
    if (r->y<y) {
        r->h-=y-r->y;
        r->y=y;
    }
    if (x+w<r->x+r->w)
        r->w=x+w-r->x;
    if (y+h<r->y+r->h)
        r->h=y+h-r->y;

    return true;
}

inline bool insideRect(const SDL_Rect & r1,const SDL_Rect & r2) {
    return r2.x<=r1.x && r1.x+r1.w<=r2.x+r2.w
           && r2.y<=r1.y && r1.y+r1.h<=r2.y+r2.h;
}


int breakRectangles(const SDL_Rect & ref,SDL_Rect r[4]) {
    if (intersectsRect(ref.x,ref.y,ref.x+ref.w,ref.y+ref.h,
                       r[0].x,r[0].y,r[0].x+r[0].w,r[0].y+r[0].h)) {
        SDL_Rect r0=r[0];
        int i=0;
        if (ref.x>r0.x) {
            r[i].x=r0.x;
            r[i].y=r0.y;
            r[i].w=r0.x+r0.w-ref.x;
            r[i].h=r0.h;
            i++;
        }
        if (ref.x+ref.w<r0.x+r0.w) {
            r[i].x=r0.x+r0.w;
            r[i].y=r0.y;
            r[i].w=r0.x+r0.w-(ref.x+ref.w);
            r[i].h=r0.h;
            i++;
        }
        if (ref.y>r0.y) {
            r[i].x=r0.x;
            r[i].y=r0.y;
            r[i].w=ref.w;
            r[i].h=r0.y+r0.h-ref.y;
            i++;
        }
        if (ref.y+ref.h<r0.y+r0.h) {
            r[i].x=r0.x;
            r[i].y=r0.y+r0.h;
            r[i].w=ref.w;
            r[i].h=r0.y+r0.h-(ref.y+ref.h);
            i++;
        }
        return i;
    } else return 1;
};


void addRect(SDL_Rect * r,const SDL_Rect & s) {
    if (s.x+s.w>r->x+r->w) 
		r->w=s.x+s.w-r->x;
    if (s.x<r->x) {
        r->w+=r->x-s.x;
        r->x=s.x;
    }
    if (s.y+s.h>r->y+r->h) 
		r->h=s.y+s.h-r->y;
    if (s.y<r->y) {
        r->h+=r->y-s.y;
        r->y=s.y;
    }
};

void bbFromLine(SDL_Rect * r,int x1,int y1,int x2,int y2) {
    if (x1<x2) {
        r->x=x1;
        r->w=x2-x1+1;
    } else {
        r->x=x2;
        r->w=x1-x2+1;
    }

    if (y1<y2) {
        r->y=y1;
        r->h=y2-y1+1;
    } else {
        r->y=y2;
        r->h=y1-y2+1;
    }
};
