/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CLIPPING_H
#define CLIPPING_H

#include "vector2d.h"
#include "SDL.h"

#define point_inside(w,h,x,y) (0<=(x) && (x)<(w) && 0<=(y) && (y)<(h))

/** 
 * Clips a segment from *v1 to *v2 with the rectangle 
 * (0,0), (w,0), (w,h), (0,h).
 *
 * The result is stored in v1 and v2.
 * @return true iff some part of the segment intersects the rectangle. 
 * */
bool clipSegment(int w,int h,Vector2d * v1,Vector2d * v2);

/**
 * @param r the rectangle to clip
 * @returns true if some part of the rectangle is visible
 */
bool clipRectangle(SDL_Rect*r,int x,int y,int w,int h);

/**
 * Breaks a rectangle in pieces which do not overlap another reference
 * rectangle.
 * @param reference reference rectangle.
 * @param r array of length four. The first positions contains the
 *     rectangle to break.
 * @returns the amount of piece rectangles.
 *          Returns 0 if the reference rectangle contains the original.
 */
int breakRectangles(const SDL_Rect & reference,SDL_Rect r[4]);

/** 
 * Calculates the bounding rectangle of *r and s. 
 * The result is returned in r.
 * */
void addRect(SDL_Rect * r,const SDL_Rect & s);

/** 
 * Computes the bounding box for a given segment (x1,y1), (x2,y2), the result
 * is placed in r.
 * */
void bbFromLine(SDL_Rect * r,int x1,int y1,int x2,int y2);

#endif // CLIPPING_H
