/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "collisionhandler.h"
#include <string>
#include "gamebody.h"
#include "starship.h"
#include "simulation.h"
#include "soundcollection.h"

extern SoundCollection * sounds;

using namespace std;

#define is(x,t) (std::string::npos!=static_cast<GameBody*>(x)->type.find(t))

void CollisionHandler::evtCollision(Simulation * sim,PhysicBody * c1,PhysicBody * c2,const Vector2d & normal) {
    //if (swept_collision(static_cast<GameBody*>(c1),static_cast<GameBody*>(c2)))
    if (!_handle_collision(sim,c1,c2,normal)) _handle_collision(sim,c2,c1,normal);
};


bool CollisionHandler::_handle_collision(Simulation * sim,PhysicBody * c1,PhysicBody * c2,const Vector2d & normal) {
    if (is(c1,"starship1")) {
        GameBody* t=dynamic_cast<GameBody*>(c2);
        if (t && t->isTrash()) {
            Starship * s=static_cast<Starship*>(c1);
            s->collectTrash(t);
            if (sounds) sounds->trashCollected.play();
            if (w) {
                w->addBonus(10);
                w->other=static_cast<GameBody*>(c1);
            }
            sim->deleteBody(c2->id);
            delete c2;
        } else {
            if (sounds) sounds->desintegrationFall.play();
            if (w) w->other=static_cast<GameBody*>(c2);
            sim->deleteBody(c1->id);
            delete c1;
        }
        return true;
    } else {
        GameBody* t=dynamic_cast<GameBody*>(c1);
        if (t && t->isTrash()) {
            if (!is(c2,"trash") && !is(c2,"starship1")) {
                if (sounds) sounds->desintegrationFall.play();

                if (w) w->other=static_cast<GameBody*>(c2);
                sim->deleteBody(c1->id);
                delete c1;
                return true;
            }
        } else if (is(c1,"asteroid")) {
            if ((is(c2,"star") || is(c2,"planet")) && !is(c2,"starship")) {
                if (w) w->other=static_cast<GameBody*>(c2);
                sim->deleteBody(c1->id);
                delete c1;
                return true;
            } /*else if (is(c2,"asteroid")) {
	// Reflect relative velocities against normal.
	Matrix2x3 m(normal.y*normal.y-normal.x*normal.x, -2*normal.y*normal.x, 0,
                    -2*normal.y*normal.x, normal.x*normal.x-normal.y*normal.y, 0);
	Vector2d kvel=m*(c1->getVelocity()-c2->getVelocity())*0;
	// Solve the system
        //   v1f=v2f+kvel
        //   (m1(v1-kvel)+m2*v2)/(m1+m2)=v2f
        // multiplying kvel changes the elasticity of the collision
	c2->setVelocity((c1->mass*(c1->getVelocity()-kvel)+c2->mass*c2->getVelocity())/(c1->mass+c2->mass));
	c1->setVelocity(c2->getVelocity()+kvel);
	return true;
      }*/
        }
    }
    return false;
};
