/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "editorcontrol.h"
#include "resourcemanager.h"
#include "serializer.h"

#include <stdio.h>
#include <sys/types.h>
#include <dirent.h>
#include <iostream>
#include <sstream>
#include <fstream>
#include <unistd.h>
#include "gcnlayouts.h"
#include "starship.h"
#include "debugmsg.h"

#define BASECOLOR (gcn::Color(90,90,90))
#define BACKCOLOR (gcn::Color(120,120,120))
#define SELECTIONCOLOR (gcn::Color(90,90,90))
#define BORDERCOLOR (gcn::Color(255,255,255))
#define FORECOLOR BORDERCOLOR


void  EditObjectControl::action (const gcn::ActionEvent & e) {
    action(e.getId());
}

void  EditObjectControl::action (const std::string& eventId) {
    if (eventId=="setmass") {
        if (ec->canvas.selected) {
            char b[10];
            double v=sldMass.getValue();
            if (sldMass.getScaleEnd()<v) v=sldMass.getScaleEnd();
            else if (v<sldMass.getScaleStart()) v=sldMass.getScaleStart();
            snprintf(b,10,"%i",int(v));
            lsetmass.setCaption(string("1e")+b);
            ec->canvas.setMass(ec->canvas.selected,pow(10,v));
            if (!updating && dynamic_cast<Starship*>(ec->canvas.selected))
                update(ec->canvas.selected);
        }
    } else if (eventId=="setfuelcapacity") {
        Starship * ship=dynamic_cast<Starship*>(ec->canvas.selected);
        if (ship) {
            char b[10];
            double v=sldFuelCapacity.getValue();
            if (sldFuelCapacity.getScaleEnd()<v)
                v=sldFuelCapacity.getScaleEnd();
            else if (v<sldFuelCapacity.getScaleStart())
                v=sldFuelCapacity.getScaleStart();
            snprintf(b,10,"%.2f",v);
            lsetfuelcap.setCaption(b);
            ship->maxFuel=v;
        }
    } else if (eventId=="setfuel") {
        Starship * ship=dynamic_cast<Starship*>(ec->canvas.selected);
        if (ship) {
            char b[10];
            double v=sldFuel.getValue();
            if (ship && ship->maxFuel<v)
                v=ship->maxFuel;
            else if (v<sldFuel.getScaleStart())
                v=sldFuel.getScaleStart();
            snprintf(b,10,"%.2f",v);
            lsetfuel.setCaption(b);
            ship->setFuel(v);
            if (!updating)
                update(ship);
        }
    } else if (eventId=="setfixed") {
        if (ec->canvas.selected) {
            if (cbFixed.isSelected())
                ec->canvas.selected->flags|=PhysicBody::FIXED;
            else 
                ec->canvas.selected->flags&=~PhysicBody::FIXED;

            ec->canvas.level->retrace();
            ec->canvas.level->step(0);
        }
    } else {
        CHERROR<<"Warning: Do not know anything about "<<eventId<<ENDL;
        exit(1);
    }
}


EditObjectControl::EditObjectControl(EditorControl * ec) :
        ec(ec),l1("Mass"),lsetmass("1e000"),lsetfuelcap("00000"),lsetfuel("00000"),
        fuelcapacity("Fuel capacity"),fuel("Fuel"),
        sl(5,5),sldMass(0,40),sldFuelCapacity(0,1000),sldFuel(0,1000),
        cbFixed("Fixed"), updating(false) {
    setOpaque(false);
    sldMass.setDimension(gcn::Rectangle(0,0,Graphic::toScreenw(70),Graphic::toScreenh(10)));
    sldMass.setBaseColor(BASECOLOR);
    sldFuelCapacity.setDimension(gcn::Rectangle(0,0,Graphic::toScreenw(70),Graphic::toScreenh(10)));
    sldFuelCapacity.setBaseColor(BASECOLOR);
    sldFuel.setDimension(gcn::Rectangle(0,0,Graphic::toScreenw(70),Graphic::toScreenh(10)));
    sldFuel.setBaseColor(BASECOLOR);
    sldMass.setValue(0);
    sldMass.addKeyListener(&ec->canvas);
    sldFuelCapacity.addKeyListener(&ec->canvas);
    sldFuel.addKeyListener(&ec->canvas);
    cbFixed.addKeyListener(&ec->canvas);

    sl.l=5;
    clvl.add(&l1);
    clvl.add(&sldMass);
    sldMass.setActionEventId("setmass");
    sldMass.addActionListener(this);
    clvl.add(&lsetmass);
    clvl.add(&cbFixed);
    cbFixed.setActionEventId("setfixed");
    cbFixed.addActionListener(this);
    cbFixed.setBackgroundColor(BACKCOLOR);
    cbFixed.setForegroundColor(FORECOLOR);

    shipcl.add(&fuelcapacity);
    shipcl.add(&sldFuelCapacity);
    sldFuelCapacity.setActionEventId("setfuelcapacity");
    sldFuelCapacity.addActionListener(this);
    shipcl.add(&lsetfuelcap);
    shipcl.setVisible(false);
    shipcl.fitWidgets();
    clvl.add(&shipcl);
    shiptrashcl.add(&fuel);
    shiptrashcl.add(&sldFuel);
    sldFuel.setActionEventId("setfuel");
    sldFuel.addActionListener(this);
    shiptrashcl.add(&lsetfuel);
    shiptrashcl.setVisible(false);
    shiptrashcl.fitWidgets();
    clvl.add(&shiptrashcl);
    sl.setContainer(&clvl);

    sl.fitWidgets();
    setContent(&sl);

    setFrameSize(1);
    int w=getScrollbarWidth()+sl.getWidth();
    setWidth(Graphic::toScreenw(115<w?115:w));
    setHeight(Graphic::toScreenh(300));
    sl.setOpaque(false);
    clvl.setOpaque(false);
    shipcl.setOpaque(false);
    shiptrashcl.setOpaque(false);
    setBaseColor(BASECOLOR);
}

void EditObjectControl::update(GameBody * selected) {
    updating=true;
    Starship * sh=dynamic_cast<Starship*>(selected);
    if (sh) {
        sldFuelCapacity.setValue(sh->maxFuel);
        action(sldFuelCapacity.getActionEventId());
        sldFuel.setValue(sh->getFuel());
        action(sldFuel.getActionEventId());
    }
    shipcl.setVisible(sh);
    shiptrashcl.setVisible(sh);

    sldMass.setValue(log10(selected->mass));
    action(sldMass.getActionEventId());
    cbFixed.setSelected(selected->flags & PhysicBody::FIXED);
    updating=false;
};

void PromptWindow::requestFocus() {
    tb.requestFocus();
};

PromptWindow::PromptWindow(string * text) : text(text), btDone("Done") {
    tb.setText(*text);
    tb.setBackgroundColor(BACKCOLOR);
    tb.setBaseColor(BASECOLOR);
    tb.setForegroundColor(FORECOLOR);

    btDone.setBackgroundColor(BACKCOLOR);
    btDone.setBaseColor(BASECOLOR);
    btDone.setForegroundColor(FORECOLOR);

    setBackgroundColor(BACKCOLOR);
    setBaseColor(BASECOLOR);
    setForegroundColor(FORECOLOR);

    sa.setBackgroundColor(BACKCOLOR);
    sa.setBaseColor(BASECOLOR);
    sa.setForegroundColor(FORECOLOR);

    vl.setBackgroundColor(BACKCOLOR);
    vl.setBaseColor(BASECOLOR);
    vl.setForegroundColor(FORECOLOR);

    btDone.addActionListener(this);
    sa.setContent(&tb);
    sa.setWidth(Graphic::toScreenw(250));
    sa.setHeight(Graphic::toScreenh(100));
    vl.add(&sa);
    vl.add(&btDone);
    vl.fitWidgets();
    add(&vl);
    setWidth(vl.getWidth());
    setHeight(vl.getHeight()+getTitleBarHeight());
}
void PromptWindow::action(const gcn::ActionEvent& ) {
    *text=tb.getText();
    setVisible(false);
}

void PromptWindow::refresh() {
    tb.setText(*text);
}

void  EditorControl::action (const gcn::ActionEvent & e) {
    action(e.getId());
}

void  EditorControl::action (const string & eventId) {
    if (eventId=="save") {
        ofstream f(patht.getText().c_str());
        f<<canvas;
    } else if (eventId=="load") {
        if (!exists(patht.getText())) {
            canvas.clear();
            return;
        }
        ifstream f(patht.getText().c_str());
        f>>canvas;
        ostringstream s;
        s<<canvas.level->getBonusTime();
        txtTime->setText(s.str());
    } else if (eventId=="play") {
        canvas.play = true;
        canvas.initTime();
    } else if (eventId=="delete") {
        if (d.isSelected()) canvas.mode=Canvas::DELETE;
    } else if (eventId=="edit") {
        if (e.isSelected()) {
            canvas.mode=Canvas::EDIT;
            ctlEdit->setVisible(true);
            ddObjects.setVisible(true);
        } else {
            canvas.selected=NULL;
            ctlEdit->setVisible(false);
            ddObjects.setVisible(false);
        }
    } else if (eventId=="describe") {
        desc.setVisible(true);
        desc.requestFocus();
        desc.refresh();
        moveToTop(&desc);
    } else if (eventId=="ddObjects") {
        canvas.selectedType=lobjects[ddObjects.getSelected()];
    }
}

bool EditorControl::exists(const string & f) {
    return !access(f.c_str(),F_OK);
    /*    DIR * d=opendir(".");
        struct dirent *ep;
        while ((ep=readdir(d)) && ep->d_name!=f);
        closedir(d);
        return ep;
    */
}

void setTxtTime(EditorControl * ec,int n) {
    ec->canvas.level->setBonusTime(n);
}


void EditorControl::initializeAddButtons(int x,int y) {
    //  const int rbCount=13;
    lobjects.push_back("player1");
    lobjects.push_back("asteroid1");
    lobjects.push_back("asteroid2");
    lobjects.push_back("asteroid3");
    lobjects.push_back("asteroid4");
    lobjects.push_back("moon1");
    lobjects.push_back("moon2");
    lobjects.push_back("moon3");
    lobjects.push_back("moon4");
    lobjects.push_back("planet1");
    lobjects.push_back("planet2");
    lobjects.push_back("planet3");
    lobjects.push_back("planet4");
    lobjects.push_back("planet5");
    lobjects.push_back("planet6");
    lobjects.push_back("planet7");
    lobjects.push_back("planet8");
    lobjects.push_back("planet9");
    lobjects.push_back("star1");
    lobjects.push_back("star2");
    lobjects.push_back("star3");
    lobjects.push_back("galaxia0");
    lobjects.push_back("galaxia0grande");
    lobjects.push_back("galaxia1");
    lobjects.push_back("galaxia2");
    lobjects.push_back("galaxia3");
    lobjects.push_back("galaxia4");
    lobjects.push_back("galaxia4chica");
    lobjects.push_back("galaxia4grande");
    lobjects.push_back("nebula1");
    lobjects.push_back("nebula2");
    lobjects.push_back("nebula3");
    lobjects.push_back("farstar1");
    lobjects.push_back("farstar2");
    lobjects.push_back("farstar3");
    lobjects.push_back("farstargroup1");
    lobjects.push_back("farstargroup2");
    lobjects.push_back("farstargroup3");
    lobjects.push_back("trash1");
    lobjects.push_back("trash2");
    lobjects.push_back("trash3");
    lobjects.push_back("trash4");
    lobjects.push_back("trash5");
    lobjects.push_back("trash6");
    lobjects.push_back("trash7");
    lobjects.push_back("trash8");
    lobjects.push_back("trash9");

    ddObjects.setWidth(Graphic::toScreenw(90));
    ddObjects.setBackgroundColor(BACKCOLOR);
    ddObjects.setSelectionColor(SELECTIONCOLOR);
    ddObjects.setBaseColor(BASECOLOR);
    ddObjects.setListModel(&lobjects);
    ddObjects.addKeyListener(&canvas);
    ddObjects.addActionListener(this);
    ddObjects.setActionEventId("ddObjects");
    action("ddObjects");
    add(&ddObjects,x,y);
}


void EditorControl::initializeModeButtons(int x,int y) {
    rbsMode.spacing=3;
    txtTime->setWidth(Graphic::toScreenw(40));
    txtTime->setHeight(Graphic::toScreenh(15));
    txtTime->setText("300");
    txtTime->setBackgroundColor(BACKCOLOR);
    txtTime->setForegroundColor(FORECOLOR);
    ltime.add(&*txtTime);
    ltime.fitWidgets();
    ltime.setBaseColor(BASECOLOR);
    save.addActionListener(this);
    save.setActionEventId("save");
    save.setBaseColor(BASECOLOR);
    load.addActionListener(this);
    load.setActionEventId("load");
    load.setBaseColor(BASECOLOR);
    btDesc.setActionEventId("describe");
    btDesc.setBaseColor(BASECOLOR);
    btDesc.addActionListener(this);
    play.setAlignment(gcn::Graphics::CENTER);
    play.adjustSize();
    play.setActionEventId("play");
    play.addActionListener(this);
    play.setBaseColor(BASECOLOR);
    d.setActionEventId("delete");
    d.addKeyListener(&canvas);
    d.addActionListener(this);
    e.setActionEventId("edit");
    e.addKeyListener(&canvas);
    e.addActionListener(this);
    play.addKeyListener(&canvas);
    save.addKeyListener(&canvas);
    load.addKeyListener(&canvas);

    d.setBackgroundColor(BACKCOLOR);
    d.setForegroundColor(FORECOLOR);
    e.setBackgroundColor(BACKCOLOR);
    e.setForegroundColor(FORECOLOR);

    rbsMode.add(&d);
    rbsMode.add(&e);
    rbsMode.setBackgroundColor(BASECOLOR);
    rbsMode.setBaseColor(BORDERCOLOR);
    bts1.add(&btDesc);
    bts1.add(&play);
    bts1.fitWidgets();
    rbsMode.add(&bts1);
    bts2.add(&save);
    bts2.add(&load);
    bts2.fitWidgets();
    rbsMode.add(&bts2);
    rbsMode.add(&patht);
    patht.setBackgroundColor(BACKCOLOR);
    patht.setForegroundColor(FORECOLOR);
    rbsMode.add(&ltime);
    rbsMode.fitWidgets();
    rbsMode.setFrameSize(1);
    rbsMode.setX(x);
    rbsMode.setY(y);
    rbsMode.setOpaque(false);
}

void EditorControl::initializeEditButtons(int x,int y) {
    ctlEdit=new EditObjectControl(this);
    ctlEdit->setX(x);
    ctlEdit->setY(y);
}

EditorControl::EditorControl():
        SpacingLayout(5,5,5,5),
        d("Delete","mode"),
        e("Edit","mode"),play("Play"),btDesc("Describe"),save("Save"),load("Load"),
        patht("unamedlevel"), txtTime(numTextField(this,setTxtTime)),
        ltime("Btime: "),
        canvas(this), desc(&canvas.level->description) { //, backgroundFill(*(new gcn::Container()))
    setBaseColor(BASECOLOR);

    desc.setCaption("Brief level description");
    add(&desc,Graphic::toScreenw(50),Graphic::toScreenh(50));
    desc.setVisible(false);
    setOpaque(false);

    setDimension(gcn::Rectangle(Graphic::toScreenx(0), Graphic::toScreeny(0), 
                                Graphic::toScreenw(LOGICAL_WIDTH), Graphic::toScreenh(LOGICAL_HEIGHT)));

    canvas.setBaseColor(gcn::Color(0,0,0));
    canvas.setDimension(gcn::Rectangle(0,0,Graphic::toScreenw(LOGICAL_WIDTH), Graphic::toScreenh(LOGICAL_HEIGHT)));
    //  canvas.setDimension(gcn::Rectangle(125, 10, 510, 465));
    add(&canvas);

    addKeyListener(&canvas);

    initializeModeButtons(Graphic::toScreenw(5),Graphic::toScreenh(10));
    initializeAddButtons(Graphic::toScreenw(5),rbsMode.getHeight()+rbsMode.getY()+Graphic::toScreenh(5));
    initializeEditButtons(Graphic::toScreenw(5),rbsMode.getHeight()+rbsMode.getY()+Graphic::toScreenh(5)+ddObjects.getHeight());

    add(&rbsMode);
    add(ctlEdit);

    e.setSelected(true),
    setFocusable(true);
}

EditorControl::~EditorControl() {
};
