/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef EDITORCONTROL_H
#define EDITORCONTROL_H

#include "canvas.h"
#include <guichan.hpp>
#include <guichan/sdl.hpp>
#include "gcnlayouts.h"
#include <sstream>
#include "reference.h"

class EditObjectControl;
class EditorControl;

class PromptWindow : public gcn::Window , public gcn::ActionListener {
private:
    std::string * text;
    CenterVerticalLayout vl;
    gcn::ScrollArea sa;
    UCSTextBox tb;
    gcn::Button btDone;
public:
    PromptWindow(string * text);
    void action(const gcn::ActionEvent &);
    void refresh();
    void requestFocus();
};

class ARadioButton : public gcn::RadioButton {
public:
    ARadioButton(const string & caption,const string& group) :
            gcn::RadioButton(caption,group) {}
    void setSelected(bool marked) {
        gcn::RadioButton::setSelected(marked);
        distributeActionEvent();
    }
};

/**
 * This class abstracts away the details of the GUI.
 * Instead of subclassing a widget
 */
class EditorControl : public SpacingLayout , public gcn::MouseListener,
            public gcn::ActionListener {

public:
    ARadioButton d,e;
    gcn::Button play,btDesc,save,load;
    gcn::TextField patht;
    Ref<gcn::TextField> txtTime;
    LabelLeft ltime;

    /**
     * Constructs an editor control.
     * @param input This is the input to sent SDL events to.
     */
    EditorControl();
    void  action (const gcn::ActionEvent & );
    void  action (const string & eventId);

    /**
     * Destroy the editor.
     */
    ~EditorControl();

    friend class EditObjectControl;
    friend class Canvas;

    LeftVerticalLayout rbsMode;
    CenterHorizontalLayout bts1;
    CenterHorizontalLayout bts2;
    EditObjectControl * ctlEdit;
    StlVectorModel lobjects;
    gcn::DropDown ddObjects;

    Canvas canvas;

private:
    void initializeAddButtons(int x,int y);
    void initializeModeButtons(int x,int y);
    void initializeEditButtons(int x,int y);
    PromptWindow desc;
    bool exists(const string & f);
    gcn::Container backgroundFill;
};

template <class Data,class SetFunction>
class NumTextField : public gcn::TextField, gcn::FocusListener {
private:
    Data d;
    SetFunction f;
public:
    NumTextField(Data d,SetFunction f,int n=0);
    void focusLost(const Event&);
};

template <class Data,class SetFunction>
NumTextField<Data,SetFunction>::NumTextField(Data d,SetFunction f,int n) :
        d(d), f(f) {
    std::ostringstream os;
    os<<n;
    setText(os.str());
    addFocusListener(this);
};
template <class Data,class SetFunction>
void NumTextField<Data,SetFunction>::focusLost(const Event&) {
    std::string t=getText();
    for (int i=0;i<t.length();i++) {
        if (!isspace(t[i]) && !isdigit(t[i])) {
            setText("0");
            break;
        }
    }
    f(d,strtol(t.c_str(),NULL,10));
};
template <class Data,class SetFunction>
inline NumTextField<Data,SetFunction> * numTextField(Data d,SetFunction f,int n=0) {
    return new NumTextField<Data,SetFunction>(d,f,n);
};

class EditObjectControl : public gcn::ScrollArea, public gcn::ActionListener {
private:
    EditorControl * ec;
    LeftVerticalLayout clvl;
    LeftVerticalLayout shipcl,shiptrashcl;
    gcn::Label l1,lsetmass,lsetfuelcap,lsetfuel,fuelcapacity,fuel;
    bool updating;
public:
    SpacingLayout sl;
    NKSlider sldMass, sldFuelCapacity, sldFuel;
    gcn::CheckBox cbFixed;

    EditObjectControl(EditorControl * ec);
    void update(GameBody*);
    void action(const gcn::ActionEvent & e);
    void  action (const std::string& eventId);
};

#endif // EDITORCONTROL_H
