/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef FILECODING_H
#define FILECODING_H

#include <iostream>
#include <sstream>
#include <string>
#include "global.h"

/** convers a char* to a basic_string<wchar_t>. */
std::basic_string<wchar_t> stows(const char * s);
std::basic_string<wchar_t> stows(const std::string & s);
/** convers a basic_string<wchar_t> to a string. */
std::string wstos(const std::basic_string<wchar_t> & s);


class Filler {
public:
    virtual size_t fill(char * buffer,size_t atmost)=0;
    virtual ~Filler() {};
};

class IStreamUTF8Decoder {
public:
    IStreamUTF8Decoder(Filler * f);

    /** if eof() then error
     *    else next character
     * @param c memory to place the result of the next character.
     * @param atmost is the amount of bytes available in c.
     * @return 0 the amount of bytes written to c
     */
    size_t read(wchar_t * c,size_t atmost);
    bool eof();
    bool error();
private:
    Filler * f;
    static const size_t cbufsize=4096;
    char cbuffer[cbufsize];
    char * cbufpos;
    size_t inbytesleft;
    int readchars;
    bool _error;
    int state;
    void refill_inputbuffer();
    IStreamUTF8Decoder & operator = (const IStreamUTF8Decoder &) {
        return *this;
    };
};


class Printer {
public:
    virtual size_t put(char * buffer,size_t atmost)=0;
    virtual ~Printer() {};
};

class OStreamUTF8Encoder {
public:
    OStreamUTF8Encoder(Printer * p);

    size_t write(const wchar_t * buffer,size_t bufsize);
    template <class S>
    OStreamUTF8Encoder & operator << (const S& a);
    int error();
private:
    Printer * p;
    static const size_t obufsize=4096;
    char obuffer[obufsize];
    int myerrno;
    OStreamUTF8Encoder & operator = (const OStreamUTF8Encoder &) {
        return *this;
    };
};

class STDStreamFiller : public Filler {
private:
    std::istream & i;
public:
    STDStreamFiller(std::istream & i);
    size_t fill(char * buffer,size_t atmost);
};

class StringFiller : public Filler {
private:
    std::string s;
public:
    StringFiller(const std::string &);
    StringFiller(const std::basic_string<wchar_t> &str);
    size_t fill(char * buffer,size_t atmost);
};

class STDStreamPrinter : public Printer {
private:
    std::ostream & o;
public:
    STDStreamPrinter(std::ostream & o);
    size_t put(char * buffer,size_t bufsize);
};

template <class S>
OStreamUTF8Encoder & OStreamUTF8Encoder::operator<< (const S& a) {
    std::ostringstream o;
    o<<a;
    *this<<stows(o.str());
    return *this;
};

template <>
inline OStreamUTF8Encoder & OStreamUTF8Encoder::operator<< <std::basic_string<wchar_t> >(const std::basic_string<wchar_t> & a) {
    write(a.data(),a.length());
    return *this;
};

template <>
inline OStreamUTF8Encoder & OStreamUTF8Encoder::operator<< <std::string>(const std::string & a) {
    *this<<stows(a);
    return *this;
};


template <>
inline OStreamUTF8Encoder & OStreamUTF8Encoder::operator<< <char>(const char & a) {
    wchar_t w=btowc(a);
    write(&w,1);
    return *this;
};


#endif // FILECODING_H
