/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "fontrenderer.h"
#include "graphic.h"
#include "guichan.hpp"

std::string::size_type findLineBegining(const std::string & str,std::string::size_type init) {
    if (init!=std::string::npos && str.length()>init)
        if (str[init]=='\n')
            if (str.length()>++init)
                if (str[init]=='\r')
                    if (str.length()>++init) return init;
                    else return std::string::npos;
                else return init;
            else return std::string::npos;
        else return init;
    else return std::string::npos;
};

void FontRenderer::drawLineL(int x, int y,const std::string & text) {
    drawLine(Graphic::toScreenx(x),Graphic::toScreeny(y),text);
}

void FontRenderer::drawL(int x, int y,const std::string & text,Alingment a) {
    draw(Graphic::toScreenx(x),Graphic::toScreeny(y),text,a);
}

void FontRenderer::draw(int x, int y,const std::string & text,Alingment a) {
    std::string::size_type init=text.length()>0 ? 0 : std::string::npos;
    int h=getHeight();
    while (init!=std::string::npos) {
        std::string::size_type end=text.find('\n',init);
        int tempx;
        std::string temptext;
        if (end!=std::string::npos) {
            temptext=text.substr(init,end-init);
            init=findLineBegining(text,end);
        } else {
            temptext=text.substr(init);
            init=end;
        }
        int width=getWidth(temptext);
        switch (a) {
        case LEFT:
            tempx=x;
            break;
        case RIGHT:
            tempx=x-width;
            break;
        default:
            tempx=x-width/2;
        }
        drawLine(tempx,y,temptext);
        SDL_Rect r={tempx,y,width+5,h};
        if (Graphic::isUpdating()) 
            Graphic::mUpdateRect(Graphic::getScreen(),&r);
        y+=h+linespacing;
    }
};


void FontRenderer::drawString(gcn::Graphics *g,const std::string & text,int x, int y) {
    x+=g->getCurrentClipArea().xOffset;
    y+=g->getCurrentClipArea().yOffset;
    drawLine(x,y,text);
};
