/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef FONTRENDERER_H
#define FONTRENDERER_H

#include <string>
#include <guichan/font.hpp>
#include "global.h"
#include "SFont.h"
#include "debugmsg.h"

class FontRenderer : public gcn::Font {
public:
    SDL_Surface *s;
    int linespacing;
    FontRenderer() : s(NULL), linespacing(0) {};
    typedef enum { LEFT, RIGHT, CENTER } Alingment;
    /** Draws a (possibly multiline) string taking the position as logical coordinates. */
    void drawL(int x, int y,const std::string & text,Alingment a=LEFT);
    /** Draws a (possibly multiline) string taking the position as screen coordinates. */
    void draw(int x, int y,const std::string & text,Alingment a=LEFT);
    void drawString(gcn::Graphics * graphics,const std::string & text,int x, int y);
    /** Draws a line taking the position in logical coordinates. */
    void drawLineL(int x, int y,const std::string & text);
    /** Draws a line taking the position as screen coordinates. */
    virtual void drawLine(int x, int y,const std::string & text)=0;
    virtual ~FontRenderer() {};
};


class SFontRenderer : public FontRenderer {
private:
    SFont_Font * f;
    void drawLine(int x, int y,const std::string & text) {
        SFont_Write(s?s:SDL_GetVideoSurface(),&*f,x,y,text.c_str());
    };
    SFontRenderer(const SFontRenderer &): FontRenderer() {};
    SFontRenderer & operator = (const SFontRenderer &) {
        return *this;
    };
public:
    SFontRenderer(SFont_Font * f) : f(f) {};
    ~SFontRenderer() {
        SFont_FreeFont(f);
    }
    int getHeight() const {
        return SFont_TextHeight(f);
    };
    int getWidth(const std::string & text) const {
        return SFont_TextWidth(f,text.c_str());
    };
};

#endif // FONTRENDERER_H
