/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef FUNCTIONEXTS_H
#define FUNCTIONEXTS_H

#include <functional>
#include <iterator>
#include "reference.h"

/**
 * Functions with three arguments
 */

template<typename arg1,typename arg2,typename arg3,typename result>
class function3  {
public:
    typedef arg1 first_argument_type;
    typedef arg2 second_argument_type;
    typedef arg3 third_argument_type;
    typedef result result_type;
};


template<class AdaptableFunction3>
class binder1st3 : public std::binary_function<typename AdaptableFunction3::second_argument_type,
            typename AdaptableFunction3::third_argument_type,
            typename AdaptableFunction3::result_type> {
public:
    binder1st3(const AdaptableFunction3& f,const typename AdaptableFunction3::first_argument_type & a) : f3(f),a1(a) {};
    typename AdaptableFunction3::result_type operator()(const typename AdaptableFunction3::second_argument_type & a2,const typename AdaptableFunction3::third_argument_type & a3) const {
        return f3(a1,a2,a3);
    }
private:
    AdaptableFunction3 f3;
    typename AdaptableFunction3::first_argument_type a1;
};

template <class AdaptableFunction3,class T>
inline binder1st3<AdaptableFunction3> bind1st3(const AdaptableFunction3 & f,const T & a) {
    return binder1st3<AdaptableFunction3> (f,a);
};

template<class AdaptableFunction3>
class binder2nd3 : public std::binary_function<typename AdaptableFunction3::first_argument_type,
            typename AdaptableFunction3::third_argument_type,
            typename AdaptableFunction3::result_type> {
private:
    AdaptableFunction3 f3;
    typename AdaptableFunction3::second_argument_type a2;
public:
    binder2nd3(const AdaptableFunction3 &f,const typename AdaptableFunction3::second_argument_type & a) : f3(f), a2(a) {};
    typename AdaptableFunction3::result_type operator() (const typename AdaptableFunction3::first_argument_type & a1,const typename AdaptableFunction3::third_argument_type & a3) const {
        return f3(a1,a2,a3);
    }
};

template <class AdaptableFunction3,class T>
inline binder2nd3<AdaptableFunction3> bind2nd3(const AdaptableFunction3 & f,const T & a) {
    return binder2nd3<AdaptableFunction3>(f,a);
};


template<class AdaptableFunction3>
class binder3rd : public std::binary_function<typename AdaptableFunction3::first_argument_type,
            typename AdaptableFunction3::second_argument_type,
            typename AdaptableFunction3::result_type> {
public:
    binder3rd(const AdaptableFunction3 & f,const typename AdaptableFunction3::third_argument_type & a) : f3(f),a3(a) {};
    typename AdaptableFunction3::result_type operator() (const typename AdaptableFunction3::first_argument_type & a1,const typename AdaptableFunction3::second_argument_type & a2)  const {
        return f3(a1,a2,a3);
    }
private:
    typename AdaptableFunction3::third_argument_type a3;
    AdaptableFunction3 f3;
};

template <class AdaptableFunction3,class T>
inline binder3rd<AdaptableFunction3> bind3rd3(const AdaptableFunction3 & f,const T & a) {
    return binder3rd<AdaptableFunction3>(f,a);
};

template<class result, class T, class arg1, class arg2>
class member_fun2 : public function3<T*,arg1,arg2,result> {
public:
    member_fun2(result (T::*f)(arg1,arg2)) : member(f) {};
    result operator() (T * t,const arg1 & a1,const arg2 & a2) const {
        return (t->*member)(a1,a2);
    };
private:
    result (T::*member)(arg1,arg2);
};

template<class result, class T, class arg1, class arg2>
inline member_fun2<result,T,arg1,arg2> mem_fun2(result (T::*f)(arg1,arg2)) {
    return member_fun2<result,T,arg1,arg2>(f);
};

/**
 * Four arguments
 */

template<typename arg1,typename arg2,typename arg3,typename arg4,typename result>
class function4  {
public:
    typedef arg1 first_argument_type;
    typedef arg2 second_argument_type;
    typedef arg3 third_argument_type;
    typedef arg4 fourth_argument_type;
    typedef result result_type;
};


template<class AdaptableFunction4>
class binder2nd4 : public function3<typename AdaptableFunction4::first_argument_type,
            typename AdaptableFunction4::third_argument_type,
            typename AdaptableFunction4::fourth_argument_type,
            typename AdaptableFunction4::result_type> {
private:
    AdaptableFunction4 f4;
    typename AdaptableFunction4::second_argument_type a2;
public:
    binder2nd4(const AdaptableFunction4 &f,const typename AdaptableFunction4::second_argument_type & a) : f4(f), a2(a) {};
    typename AdaptableFunction4::result_type operator() (const typename AdaptableFunction4::first_argument_type & a1,const typename AdaptableFunction4::third_argument_type & a3,const typename AdaptableFunction4::fourth_argument_type & a4) const {
        return f4(a1,a2,a3,a4);
    }
};

template<class AdaptableFunction4,class T>
inline binder2nd4<AdaptableFunction4> bind2nd4(const AdaptableFunction4 & f,const T & a) {
    return binder2nd4<AdaptableFunction4>(f,a);
};

template<class AdaptableFunction4>
class binder1st4 : public function3<typename AdaptableFunction4::second_argument_type,
            typename AdaptableFunction4::third_argument_type,
            typename AdaptableFunction4::fourth_argument_type,
            typename AdaptableFunction4::result_type> {
private:
    AdaptableFunction4 f4;
    typename AdaptableFunction4::first_argument_type a1;
public:
    binder1st4(const AdaptableFunction4 &f,const typename AdaptableFunction4::first_argument_type & a) : f4(f), a1(a) {};
    typename AdaptableFunction4::result_type operator()
    (const typename AdaptableFunction4::second_argument_type & a2,
     const typename AdaptableFunction4::third_argument_type & a3,
     const typename AdaptableFunction4::fourth_argument_type & a4) const {
        return f4(a1,a2,a3,a4);
    }
};

template<class AdaptableFunction4,class T>
inline binder1st4<AdaptableFunction4> bind1st4(const AdaptableFunction4 & f,const T & a) {
    return binder1st4<AdaptableFunction4>(f,a);
};


template<class AdaptableFunction4>
class binder3rd4 : public function3<typename AdaptableFunction4::first_argument_type,
            typename AdaptableFunction4::second_argument_type,
            typename AdaptableFunction4::fourth_argument_type,
            typename AdaptableFunction4::result_type> {
private:
    typename AdaptableFunction4::third_argument_type a3;
    AdaptableFunction4 f4;
public:
    binder3rd4(const AdaptableFunction4 &f,const typename AdaptableFunction4::third_argument_type & a) : f4(f), a3(a) {};
    typename AdaptableFunction4::result_type operator()
    (const typename AdaptableFunction4::first_argument_type & a1,
     const typename AdaptableFunction4::second_argument_type & a2,
     const typename AdaptableFunction4::fourth_argument_type & a4) const {
        return f4(a1,a2,a3,a4);
    }
};

template<class AdaptableFunction4,class T>
inline binder3rd4<AdaptableFunction4> bind3rd4(const AdaptableFunction4 & f,const T & a) {
    return binder3rd4<AdaptableFunction4>(f,a);
};

template<class AdaptableFunction4>
class binder4th4 : public function3<typename AdaptableFunction4::first_argument_type,
            typename AdaptableFunction4::second_argument_type,
            typename AdaptableFunction4::third_argument_type,
            typename AdaptableFunction4::result_type> {
private:
    typename AdaptableFunction4::fourth_argument_type a4;
    AdaptableFunction4 f4;
public:
    binder4th4(const AdaptableFunction4 &f,const typename AdaptableFunction4::fourth_argument_type & a) : f4(f), a4(a) {};
    typename AdaptableFunction4::result_type operator()
    (const typename AdaptableFunction4::first_argument_type & a1,
     const typename AdaptableFunction4::second_argument_type & a2,
     const typename AdaptableFunction4::third_argument_type & a3) const {
        return f4(a1,a2,a3,a4);
    }
};

template<class AdaptableFunction4,class T>
inline binder4th4<AdaptableFunction4> bind4th4(const AdaptableFunction4 & f,const T & a) {
    return binder4th4<AdaptableFunction4>(f,a);
};


template<class result, class T, class arg1, class arg2, class arg3>
class member_fun3 : public function4<T*,arg1,arg2,arg3,result> {
public:
    member_fun3(result (T::*f)(arg1,arg2,arg3)) : member(f) {};
    result operator() (T * t,const arg1 & a1,const arg2 & a2,const arg3 & a3) const {
        return (t->*member)(a1,a2,a3);
    };
private:
    result (T::*member)(arg1,arg2,arg3);
};

template<class result, class T, class arg1, class arg2, class arg3>
inline member_fun3<result,T,arg1,arg2,arg3> mem_fun3(result (T::*f)(arg1,arg2,arg3)) {
    return member_fun3<result,T,arg1,arg2,arg3>(f);
};


template <class Iterator,class ValueType>
int indexOf(Iterator begin,Iterator end,ValueType v) {
    int counter=0;
    for (Iterator i=begin;i!=end;i++) {
        if (v==*i) return counter;
        counter++;
    }
    return -1;
};

template <class ValueType,class Iterator>
ValueType elementAt(Iterator i, int pos) {
    for (int counter=0;counter<pos;counter++) i++;
    return *i;
};

/** Iterator utilities */

template <class Value,class Iterator>
class map_iterator : public std::iterator_traits<Iterator> {
public:
    typedef Value (*f_map)(typename std::iterator_traits<Iterator>::reference);
private:
    Iterator i;
    f_map f;
public:
    map_iterator (const Iterator & i,f_map f) : i(i),f(f) {};
    map_iterator & operator ++() {
        ++i;
        return *this;
    };
    map_iterator & operator ++ (int) {
        i++;
        return *this;
    };
    bool operator == (const Iterator &ci) const {
        return i==ci;
    };
    bool operator != (const Iterator &ci) const {
        return i!=ci;
    };
    operator Iterator () const {
        return i;
    };
    Value operator * () {
        return f(*i);
    };
};

template <class Value,class Iterator>
inline map_iterator<Value,Iterator> map_it(const Iterator & i,typename map_iterator<Value,Iterator>::f_map f) {
    return map_iterator<Value,Iterator>(i,f);
};

template <class Iterator>
class filter_iterator : public std::iterator_traits<Iterator> {
public:
    typedef bool (*p_filter)(typename std::iterator_traits<Iterator>::reference);
private:
    Iterator i,end;
    p_filter p;
public:
    filter_iterator (const Iterator & i,const Iterator & end,p_filter p) : i(i),end(end),p(p) {
        while (this->i!=end && !p(*(this->i))) (this->i)++;
    };
    filter_iterator & operator ++() {
        do ++i;
        while (i!=end && !p(*i));
        return *this;
    };
    filter_iterator & operator ++ (int) {
        ++(*this);
        return *this;
    };
    bool operator == (const Iterator &ci) const {
        return i==ci;
    };
    bool operator != (const Iterator &ci) const {
        return i!=ci;
    };
    operator Iterator () const {
        return i;
    };
    typename std::iterator_traits<Iterator>::reference operator * () {
        return *i;
    };
};

template <class Iterator>
inline filter_iterator<Iterator> filter_it(const Iterator & i,const Iterator & end,typename filter_iterator<Iterator>::p_filter p) {
    return filter_iterator<Iterator>(i,end,p);
}

template <class Iterator>
class const_filter_iterator : public std::iterator_traits<Iterator> {
public:
    typedef bool (*p_filter)(const typename std::iterator_traits<Iterator>::reference);
private:
    Iterator i,end;
    p_filter p;
public:
    const_filter_iterator (const Iterator & i,const Iterator & end,p_filter p) : i(i),end(end),p(p) {
        while (this->i!=end && !p(*(this->i))) (this->i)++;
    };
    const_filter_iterator & operator ++() {
        do ++i;
        while (i!=end && !p(*i));
        return *this;
    };
    const_filter_iterator & operator ++ (int) {
        ++(*this);
        return *this;
    };
    bool operator == (const Iterator &ci) const {
        return i==ci;
    };
    bool operator != (const Iterator &ci) const {
        return i!=ci;
    };
    operator Iterator () const {
        return i;
    };
    const typename std::iterator_traits<Iterator>::reference operator * () {
        return *i;
    };
};

template <class Iterator>
inline const_filter_iterator<Iterator> const_filter_it(const Iterator & i,const Iterator & end,typename const_filter_iterator<Iterator>::p_filter p) {
    return const_filter_iterator<Iterator>(i,end,p);
}

template <class B,class A>
B stcast(A a) {
    return static_cast<B>(a);
}

template <class Value,class Iterator>
class cast_iterator : public map_iterator <Value,Iterator> {
public:
    cast_iterator(const Iterator & i): map_iterator<Value,Iterator>(i,stcast<Value,typename std::iterator_traits<Iterator>::reference>) {};
};

template <class Value,class Iterator>
class cast_iterator<const Value,Iterator> : public map_iterator <const Value,Iterator> {
public:
    cast_iterator(const Iterator & i): map_iterator<const Value,Iterator>(i,stcast<const Value,const typename std::iterator_traits<Iterator>::reference>) {};
};

template <class Value,class Iterator>
inline cast_iterator<Value,Iterator> cast_it(const Iterator & i) {
    return cast_iterator<Value,Iterator>(i);
}

class Object {
public:
    virtual ~Object() {};
};

template <class Data,class UnaryFunction>
class OnDestruction : public Object {
private:
    Data d;
    UnaryFunction f;
public:
    OnDestruction(Data d,UnaryFunction f) : d(d),f(f) {};
    ~OnDestruction() {
        f(d);
    };
};

template <class Data,class UnaryFunction>
inline OnDestruction<Data,UnaryFunction> * on_destruction(Data d,UnaryFunction f) {
    return new OnDestruction<Data,UnaryFunction>(d,f);
};

template void del<Object>(Object*p);

template <class Container>
class erase_iterator : public std::iterator_traits<typename Container::iterator> {
public:
    typedef bool (*p_erase)(typename Container::const_reference);
private:
    static void _setFalse(bool * iterating) {
        *iterating=false;
    }
    Ref<Object> r;
    typename Container::iterator i,end;
    p_erase d;
    bool * iterating;
    bool _iterating;
    Container * c;
    erase_iterator & operator = (const erase_iterator &ei) const {
        c=ei.c;
        iterating=ei.iterating;
        _iterating=ei._iterating;
        d=ei.d;
        i=ei.i;
        end=ei.end;
        r=ei.r;
        return *this;
    };
    void search();

public:
    erase_iterator(const typename Container::iterator & i,const typename Container::iterator & end,Container * c,bool * iterating,p_erase d) :
            i(i),end(end),d(d),iterating(iterating), _iterating(*iterating),c(c) {
        if (!_iterating) {
            *iterating=true;
            r=Ref<Object>(on_destruction(iterating,&erase_iterator<Container>::_setFalse));
        }
        search();
    };
    erase_iterator & operator ++() {
        ++i;
        search();
        return *this;
    };
    erase_iterator & operator ++ (int) {
        ++(*this);
        return *this;
    };
    bool operator == (const typename Container::iterator &di) const {
        return i==di;
    };
    bool operator != (const typename Container::iterator &di) const {
        return i!=di;
    };
    bool operator == (const erase_iterator &di) const {
        return i==di.i;
    };
    bool operator != (const erase_iterator &di) const {
        return i!=di.i;
    };
    //operator Container::iterator () const { return i; };
    typename Container::value_type & operator * () {
        return *i;
    };
    typename Container::value_type * operator -> () {
        return &*i;
    };
};

template <class Container>
void erase_iterator<Container>::search() {
    if (!_iterating) {
        typename Container::iterator j;
        while (i!=end && d(*i)) {
            j=i++;
            c->erase(j);
        }
    } else while (i!=end && d(*i)) i++;
}


template <class Container>
inline erase_iterator<Container>
erase_it(const typename Container::iterator & i,const typename Container::iterator & end,Container * c,bool * iterating,typename erase_iterator<Container>::p_erase d) {
    return erase_iterator<Container>(i,end,c,iterating,d);
}

template <class ValueType,class Container>
class DeleteIterationContainer {
private:
    typename erase_iterator<Container>::p_erase get_erase;
    typename map_iterator<ValueType,typename Container::iterator>::f_map get_data;
    bool iterating;
    Container * c;
public:
    typedef erase_iterator<Container> erase_iterator_type;
    typedef map_iterator<ValueType,erase_iterator_type> iterator;
    DeleteIterationContainer(Container * c,
                             typename erase_iterator<Container>::p_erase get_erase,
                             typename map_iterator<ValueType,typename Container::iterator>::f_map get_data) :
            get_erase(get_erase), get_data(get_data), iterating(false),c(c) {};
    iterator begin() {
        return map_it<ValueType>(erase_it(c->begin(),c->end(),c,&iterating,get_erase),get_data);
    }
    erase_iterator_type end() {
        return erase_it(c->end(),c->end(),c,&iterating,get_erase);
    }
    bool isIterating() {
        return iterating;
    }
};


# endif // FUNCTIONEXTS_H
