/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "gamebody.h"
#include <iostream>

#include "parsercombinators.h"

GameBody::GameBody(real m,real x0,real y0,real vx0,real vy0,real w):
        Graphic(),PhysicBody(m,x0,y0,vx0,vy0,w),
        cmfColor(SDL_MapRGB(Graphic::getScreen()->format,255,255,255)) {}

GameBody::GameBody(const std::string & image,real m,real x0,real y0,real vx0,real vy0,real w,bool usecache,bool prerotate):
        Graphic(image,usecache,false,prerotate),PhysicBody(m,x0,y0,vx0,vy0,w),type(image),mask(image),
        cmfColor(SDL_MapRGB(Graphic::getScreen()->format,255,255,255)) {}

GameBody::GameBody(const std::string & image,Vector2d pos,Vector2d vel,real m,real w,bool usecache,bool prerotate) :
        Graphic(image,usecache,false,prerotate),PhysicBody(pos,vel,m,w),type(image),mask(image),
        cmfColor(SDL_MapRGB(Graphic::getScreen()->format,255,255,255)) {}

Matrix2x3 GameBody::get_view_transformation(bool rotate,real zoom,const Vector2d & shift) const {
    return Matrix2x3(zoom,zoom,rotate?-getAngle():0,-getPositionX()-shift.x,-getPositionY()-shift.y,screencenter.x,screencenter.y);
};

Matrix2x3 GameBody::get_view2world_transformation(bool rotate,real zoom,const Vector2d & shift) const {
    return Matrix2x3(1.0/zoom,1.0/zoom,rotate?getAngle():0,-screencenter.x,-screencenter.y,getPositionX()+shift.x,getPositionY()+shift.y);
}

PhysicBody * GameBody::clone() {
    return new GameBody(*this);
};

bool GameBody::contains(const Vector2d * point) const {
    Vector2d p=*point-(getPosition()-hotspot);
    return 0 <= p.x && p.x<=getWidth() &&
           0 <= p.y && p.y<=getHeight();
}

bool GameBody::contains(real zoom,const Vector2d * point) const {
    Vector2d p=*point-(getPosition()-hotspot*zoom);
    p.x+=5/zoom;
    p.y+=5/zoom;
    return 0 <= p.x && p.x<=getWidth()+10/zoom &&
           0 <= p.y && p.y<=getHeight()+10/zoom;
}

bool GameBody::touches(const SDL_Rect * r) {
    Vector2d p(r->x-(getPositionX()-hotspot.x),r->y-(getPositionY()-hotspot.y));
    return  0 <= p.x+r->w && p.x <= getWidth() &&
            0 <= p.y+r->h && p.y <= getHeight();
}

#define is(x,t) (std::string::npos!=static_cast<const GameBody*>(x)->type.find(t))
bool GameBody::isTrash() const {
    return is(this,"trash");
}

std::ostream & GameBody::serialize(std::ostream & o) const {
    return o<<type<<' '<<getPosition()<<' '<<getVelocity()<<' '<<w<<' '<<mass<<' '<<uint32_t(flags);
};
