/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef GAMECONTROLS_H
#define GAMECONTROLS_H

#include "inputcontrol.h"
#include "bar.h"
#include "zoomcontrol.h"
#include "resourcemanager.h"
#include "mousescroll.h"
#include <list>

class cWorld;
class Effect;

/** 
 * Implementation to play a level. 
 * Create an instance from a given cWorld instance and then call method playLevel().
 * This class implements some effects to draw text on top of the scene and to make 
 * fade ins and fade outs.
 * */
class GameControls {
public:
    GameControls(cWorld *);
    void playLevel();

    ~GameControls();

    bool done;
private:
    InputControl ic;
    int chTimeForward;

    void begin();
    bool finish();
    void draw();
    void step(int);
    bool process_events(Uint32 delta);
    void mouseMotion(int x, int y);
    std::list<Effect*> effects;
    FuelBar fuelbar;
    cWorld * w;
    ZoomControl zc;
    Button shipviewbt;
    Button viewpointbt;
    Button orbitlengthbt;
    Button timebt;
    Graphic quitbt,helpbt,scoredisplay;
    MouseScroll mscroll;
    bool active,focus,ending;
    int lastBonus;
    int lastCollected;
	bool pauseMode;
    Uint16 wheelcount;
    int wheel;
    std::string scorelabel, bonuslabel, collectlabel, helpstr;
    int scorelabelx, bonuslabelx, collectlabelx, scorex;
    friend void cycleMusic();
    RMFONTRef font,font12,font24pgray;
};

#endif // GAMECONTROLS_H
