/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "gcnlayouts.h"

Layout::Layout() {
    setOpaque(false);
}

void Layout::calculateBB() {
    int w=0,h=0,x=0,y=0;
    if (!mWidgets.empty()) {
        WidgetList::iterator i=mWidgets.begin();
        x=(*i)->getX();
        y=(*i)->getY();
        w=(*i)->getWidth();
        h=(*i)->getHeight();
        i++;
        for (;i!=mWidgets.end();i++) {
            if ((*i)->getX()<x) {
                w+=x-(*i)->getX();
                x=(*i)->getX();
            }
            if ((*i)->getY()<y) {
                h+=y-(*i)->getY();
                y=(*i)->getY();
            }
            if ((*i)->getX()+(*i)->getWidth()>x+w) w=(*i)->getX()+(*i)->getWidth()-x;
            if ((*i)->getY()+(*i)->getHeight()>y+h) h=(*i)->getY()+(*i)->getHeight()-y;
        }

        for (i=mWidgets.begin();i!=mWidgets.end();i++)
            (*i)->setPosition((*i)->getX()-x,(*i)->getY()-y);

    }
    setX(x);
    setY(y);
    setWidth(w);
    setHeight(h);
}
void Layout::fitWidgets() {
    calculateBB();
}

CenterVerticalLayout::CenterVerticalLayout(int spacing)
        : spacing(spacing) {};

void CenterVerticalLayout::calculateBB() {
    int w=0,h=0;
    if (!mWidgets.empty()) {
        WidgetList::iterator i=mWidgets.begin();
        w=(*i)->getWidth();
        h=(*i)->getHeight();
        i++;
        for (;i!=mWidgets.end();i++) {
            if (w<(*i)->getWidth()) w=(*i)->getWidth();
            h+=(*i)->getHeight()+spacing;
        }
    }
    setWidth(w);
    setHeight(h);
}
void CenterVerticalLayout::fitWidgets() {
    calculateBB();
    int y=0;
    for (WidgetList::iterator i=mWidgets.begin();i!=mWidgets.end();i++) {
        (*i)->setX(getX()+(getWidth()-(*i)->getWidth())/2);
        (*i)->setY(y);
        y+=spacing+(*i)->getHeight();
    }
}


LeftVerticalLayout::LeftVerticalLayout(int spacing)
        : CenterVerticalLayout(spacing) {};

void LeftVerticalLayout::fitWidgets() {
    calculateBB();
    int y=0;
    for (WidgetList::iterator i=mWidgets.begin();i!=mWidgets.end();i++) {
        (*i)->setX(0);
        (*i)->setY(y);
        y+=spacing+(*i)->getHeight();
    }
};

SpacingLayout::SpacingLayout(int u,int d,int l,int r):c(NULL),u(u),d(d),l(l),r(r) {};

void SpacingLayout::setContainer(Layout * cl) {
    c=cl;
    add(cl);
}

void SpacingLayout::fitWidgets() {
    if (c) {
        c->fitWidgets();
        setHeight(u+d+c->getHeight());
        setWidth(l+r+c->getWidth());
        c->setPosition(l,u);
    }
};

void CenterHorizontalLayout::calculateBB() {
    int w=0,h=0;
    if (!mWidgets.empty()) {
        WidgetList::iterator i=mWidgets.begin();
        w=(*i)->getWidth();
        h=(*i)->getHeight();
        i++;
        for (;i!=mWidgets.end();i++) {
            if (h<(*i)->getHeight()) h=(*i)->getHeight();
            w+=(*i)->getWidth()+spacing;
        }
    }
    setWidth(w);
    setHeight(h);
}

CenterHorizontalLayout::CenterHorizontalLayout(int spacing): spacing(spacing) {};
void CenterHorizontalLayout::fitWidgets() {
    calculateBB();
    int x=0;
    for (WidgetList::iterator i=mWidgets.begin();i!=mWidgets.end();i++) {
        (*i)->setY(getY()+(getHeight()-(*i)->getHeight())/2);
        (*i)->setX(x);
        x+=spacing+(*i)->getWidth();
    }
};

void UCSTextBox::keyPressed(gcn::KeyEvent& e) {
    gcn::Key key = e.getKey();
    if (key.getValue()==161 && mEditable) {
        mTextRows[mCaretRow].insert(mCaretColumn,std::string(1,(char)key.getValue()));
        ++mCaretColumn;
    }
    gcn::TextBox::keyPressed(e);
}
