/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef GCNLAYOUTS_H
#define GCNLAYOUTS_H
#include <guichan.hpp>

class Layout : public gcn::Container {
protected:
    virtual void calculateBB();
public:
    Layout();
    virtual void fitWidgets();
    virtual ~Layout() {};
};

class CenterVerticalLayout : public Layout {
protected:
    void calculateBB();
public:
    int spacing;
    CenterVerticalLayout(int spacing=0);
    void fitWidgets();
};

class LeftVerticalLayout : public CenterVerticalLayout {
public:
    LeftVerticalLayout(int spacing=0);
    void fitWidgets();
};

class CenterHorizontalLayout : public Layout {
protected:
    void calculateBB();
public:
    int spacing;
    CenterHorizontalLayout(int spacing=0);
    void fitWidgets();
};

class SpacingLayout : public Layout {
private:
    Layout * c;
public:
    int u,d,l,r;
    SpacingLayout(int u=5,int d=5,int l=5,int r=5);
    void setContainer(Layout * cl);
    void fitWidgets();
};

class LabelLeft : public CenterHorizontalLayout {
public:
    gcn::Label l;
    LabelLeft(const std::string label,int spacing=5) : CenterHorizontalLayout(spacing),l(label) {
        add(&l);
    }
};

class LabelUp : public LeftVerticalLayout {
public:
    gcn::Label l;
    LabelUp(const std::string label,int spacing=5) : LeftVerticalLayout(spacing),l(label) {
        add(&l);
        this->setOpaque(false);
    }
};

class CenterLabelUp : public CenterVerticalLayout {
public:
    gcn::Label l;
    CenterLabelUp(const std::string label,int spacing=5) : CenterVerticalLayout(spacing),l(label) {
        add(&l);
        this->setOpaque(false);
    }
};

class LabelDown : public LeftVerticalLayout {
public:
    gcn::Label l;
    LabelDown(const std::string label,int spacing=5) : LeftVerticalLayout(spacing),l(label) {
        add(&l);
    }
    void fitWidgets() {
        remove(&l);
        add(&l);
        LeftVerticalLayout::fitWidgets();
    };
};

class NKSlider : public gcn::Slider {
public:
    NKSlider(double scaleStart,double scaleEnd) : gcn::Slider(scaleStart,scaleEnd) {};
    void keyPressed(gcn::KeyEvent&) {};
};

class StlVectorModel : public std::vector<std::string>, public gcn::ListModel {
public:
    int getNumberOfElements() {
        return size();
    };
    std::string getElementAt(int i) {
        return (*this)[i];
    }
    virtual ~StlVectorModel() {};
};


class UCSTextBox : public gcn::TextBox {
    void keyPressed(gcn::KeyEvent& e);
};



#endif // GCNLAYOUTS_H

