/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "inputcontrol.h"
#include "cworld.h"
#include "starship.h"
#include "pathdrawer.h"
#include "runtimeopts.h"

//#include <iostream.h>

#define VECTOR_LENGTH 75.0

extern int mousePositionX;
extern int mousePositionY;

InputControl::InputControl(cWorld * w): w(w),
        crossCursor("mousepointer",false),
        shipcontrol("controlpanel",false) {
    mpos.x=mousePositionX;
    mpos.y=mousePositionY;
    lastMouseMove=0;
    decay=0;
}

void InputControl::step(int) {
    if (w->zoomTimev>1) {
        if (w->getShip() && w->pathDrawer) 
            w->pathDrawer->onStep();
    }
}

inline real vorientation(const Vector2d & dir) {
    if (dir.x!=0.0) return atan(dir.y/dir.x)*180.0/M_PI+90.0*(dir.x>=0?1:-1);
    else return 180.0*(dir.y>=0?1:0);
}

#define ACCBOX 119
#define TRACKBEAMBOX 136
#define TRASHFIREBOX 153
#define REDBUTTONBOX 170

void InputControl::draw() {
    UPDATEOFF(shipcontrol.draw(0,LOGICAL_HEIGHT-shipcontrol.getHeight()));
}

void InputControl::drawCursor() {
    if (decay>0) {
        crossCursor.draw(mpos);
        if (decay+lastMouseMove<Graphic::ticks()) 
			decay=0;
    };
}

void InputControl::mouseClick(int x,int y,bool left) {
    decay=500;

    if (left && !(SDL_GetModState() & KMOD_SHIFT)) { // Right button was pressed
        if (w->getShip()) {
            if (w->setTrackCoordinateSystem(x,y)) {
                if (w->pathDrawer)
                    w->pathDrawer->resetEvents();
            }
        } else
            w->setTrackCoordinateSystem(x,y);
    }
}


void InputControl::evtResetPath() {
    w->pathDrawer->emptyEventQueue();
    w->retrace();
}

void InputControl::mouseMotion(int x,int y) {
    lastMouseMove=Graphic::ticks();
    decay=500;

    mpos.x=x;
    mpos.y=y;
    if (!w->getShip()) return;
}

