/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef INPUTCONTROL_H
#define INPUTCONTROL_H

#include "graphic.h"
#include "starship.h"

class cWorld;

class ControlHandler {
public:
    virtual void evtResetPath()=0;
    virtual ~ControlHandler() {};
};

/**
 * Handles drawing and states of mouse input.
 */
class InputControl : public ControlHandler {
public:
    InputControl(cWorld * w);

    void draw();
    void drawCursor();
    void step(int);

    /**
     * Tells the input control which is the mouse position after
     * mouse moves.
     * @param x Horizontal screen coordinate of the mouse position.
     * @param y Vertical screen coordinate of the mouse position.
     */
    void mouseMotion(int x,int y);

    /**
     * Tells the input control which is the mouse position after a button
     * click event.
     * @param x Horizontal screen coordinate of the mouse position.
     * @param y Vertical screen coordinate of the mouse position.
     * @param left Whether the left or the rigth button was clicked.
     */
    void mouseClick(int x,int y,bool left);

    void evtResetPath();

private:
    /**
     * Where to draw the control center in screen coordinates.
     */
    cWorld * w;
    Vector2d mpos;
    unsigned int lastMouseMove;
    unsigned int decay;

    void oncontrol();
    Graphic crossCursor,shipcontrol;
};

#endif // INPUTCONTROL_H
