/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <guichan.hpp>
#include <guichan/sdl.hpp>
#include <SDL.h>
#include "graphic.h"
#include "editorcontrol.h"
#include <iostream>
#include <fstream>
#include "cworld.h"
#include "gamecontrols.h"
#include "pathdrawer.h"
#include "music.h"

using namespace gcn;
using namespace std;

extern int mousePositionX;
extern int mousePositionY;

extern bool optrealtime;

bool process_events(SDLInput * input,SDLGraphics * graphics,EditorControl * ec,int delta);

/*
#include <dirent.h>

void convertLevels(EditorControl* ec,const char* path){
    DIR * d=opendir(path);
    struct dirent *ep;
    while ((ep=readdir(d))) {
        if (ep->d_name[0]=='l') {
            std::string f = std::string(path)+"/"+ep->d_name;
            std::cout<<f<<std::endl;
            {
                ifstream inputf(f.c_str());
                inputf>>ec->canvas;
            }
            ofstream outputf(f.c_str());
            outputf<<ec->canvas;
        }
    }
    closedir(d);
}*/


void runLeveleditor(string & file) {
    // Get SDL working
    SDL_WM_SetCaption("Spacejunk level editor","Spacejunk level editor");
    SDL_ShowCursor(SDL_DISABLE);
    Graphic::clearScreen(SDL_MapRGB(Graphic::getScreen()->format,0,0,0));
    Graphic::flip(true);
    Graphic::setPalette("main");

    bool needResetPalette=false;

    try {
        SDLImageLoader imageLoader;
        Image::setImageLoader(&imageLoader);

        SDLGraphics graphics;
        graphics.setTarget(Graphic::getScreen());

        RMFONTRef font("12p");
        gcn::Widget::setGlobalFont(&*font);

        SDLInput input;

        EditorControl ec;
        ec.setPosition(Graphic::toScreenx(0),Graphic::toScreeny(0));
        //convertLevels(&ec,"levels");
        //convertLevels(&ec,"demolevels");
        if (file!="") {
            ifstream f(file.c_str());
            f>>ec.canvas;
            ec.patht.setText(file);
            ostringstream s;
            s<<ec.canvas.level->getBonusTime();
            ec.txtTime->setText(s.str());
        }

        Gui gui;
        gui.setGraphics(&graphics);
        gui.setInput(&input);
        gui.setTop(&ec);
        ec.requestFocus();
        Graphic mpointer("mousepointer",false);
        mpointer.hotspot=Vector2d();
        Graphic deletepointer("deletepointer",false);
        deletepointer.hotspot.x = deletepointer.getWidth()/2;
        deletepointer.hotspot.y = deletepointer.getHeight()/2;
        int last_step=Graphic::ticks();
        int delta=0;

        while (process_events(&input,&graphics,&ec,delta)) {

            if (!ec.canvas.play) {
                // Clear screen margins
                SDL_Rect blackr = {0,0,Graphic::screenx,Graphic::screenh};
                SDL_FillRect(Graphic::getScreen(),&blackr,SDL_MapRGB(Graphic::getScreen()->format,0,0,0));
                blackr.x = Graphic::screenx+Graphic::screenw;
                SDL_FillRect(Graphic::getScreen(),&blackr,SDL_MapRGB(Graphic::getScreen()->format,0,0,0));
                blackr.x = 0;
                blackr.w = Graphic::screenw;
                blackr.h = Graphic::screeny;
                SDL_FillRect(Graphic::getScreen(),&blackr,SDL_MapRGB(Graphic::getScreen()->format,0,0,0));
                blackr.y = Graphic::screeny+Graphic::screenh;
                SDL_FillRect(Graphic::getScreen(),&blackr,SDL_MapRGB(Graphic::getScreen()->format,0,0,0));

                // Draw level editor
                gui.logic();
                gui.draw();
                if (ec.canvas.mode==Canvas::EDIT)
                    mpointer.draw(mousePositionX,mousePositionY);
                else
                    deletepointer.draw(mousePositionX,mousePositionY);
                Graphic::flip();
                if (needResetPalette) {
                    Graphic::setPalette("main");
                    needResetPalette=false;
                }
                delta=Graphic::ticks()-last_step;
                if (delta<33) 
                    ec.canvas.level->extraTime(33-delta);
                else 
                    ec.canvas.level->extraTime(10);
                delta=Graphic::ticks()-last_step;
                if (delta<33)
                    SDL_Delay(33-delta);
                last_step=Graphic::ticks();
            } else {

                GameControls gc(ec.canvas.level);
                Music::setSilence(true);
                gc.playLevel();
                Music::setSilence(false);

                ec.canvas.level->pathDrawer->emptyEventQueue();
                ec.canvas.level->retrace();
                ec.canvas.level->step(0);
                ec.canvas.level->finish=false;

                ec.canvas.play=false;
                ec.canvas.refreshViewTransformation();
                ec.canvas.selected=NULL;

                needResetPalette=true;
            }
        }
        file=ec.patht.getText();
    } catch (Exception e) {
        std::cout << "Std exception: " << e.getMessage() << std::endl;
    } 
}

#define WHEELDELAY (30)
int wheel=0;
int wheelcount=0;

/**
 * Process events that affects the current level.
 * @returns int -1 if a quit event is received, 0 if escape is pressed, and 1 otherwise.
 */
bool process_events(SDLInput * input,SDLGraphics * graphics,EditorControl * ec,int delta) {
    SDL_Event e;
    while (SDL_PollEvent(&e)) {
        input->pushInput(e);
        switch (e.type) {
        case SDL_KEYDOWN:
            switch (e.key.keysym.sym) {
            case SDLK_ESCAPE:
                return false;
            default:
                ;
            }
            break;
        case SDL_KEYUP:
            switch (e.key.keysym.sym) {
            case SDLK_KP_MULTIPLY:
                ec->canvas.level->zoomTime=0.5;
                ec->canvas.level->zoomTimev=1;
                break;
            default:
                ;
            }
            break;
        case SDL_MOUSEMOTION:
            mousePositionX=Graphic::fromScreenx(e.motion.x);
            mousePositionY=Graphic::fromScreeny(e.motion.y);
            break;
        case SDL_MOUSEBUTTONDOWN:
            switch (e.button.button) {
            case SDL_BUTTON_WHEELUP:
                ec->canvas.zc.pressOut();
                if (wheel) wheelcount+=WHEELDELAY;
                else {
                    wheelcount=delta+WHEELDELAY;
                    wheel=1;
                }
                break;
            case SDL_BUTTON_WHEELDOWN:
				ec->canvas.level->zoomEye.x = mousePositionX;
				ec->canvas.level->zoomEye.y = mousePositionY;
                ec->canvas.zc.pressIn();
                if (wheel) wheelcount+=WHEELDELAY;
                else {
                    wheelcount=delta+WHEELDELAY;
                    wheel=-1;
                }
                break;
            default:
                ;
            }
        case SDL_MOUSEBUTTONUP:
            if (e.button.button!=SDL_BUTTON_WHEELDOWN &&
                    e.button.button!=SDL_BUTTON_WHEELUP) ec->canvas.zc.release();
            break;
        case SDL_ACTIVEEVENT: {
            Uint8 st=SDL_GetAppState();
            if (st & (SDL_APPINPUTFOCUS | SDL_APPACTIVE)) {
                if (!e.active.gain)
                    Music::pause();
                else
                    Music::resume();
            }
            break;
        }
        case SDL_QUIT:
            throw 0;
        default:
            ;
        }
    };

    if (wheel) {
        if (delta<=wheelcount)
            wheelcount-=delta;
        else wheelcount=0;
        if (!wheelcount) {
            ec->canvas.zc.release();
            wheel=0;
        };
    }

    return true;
}

