/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "music.h"
#include "SDL_mixer.h"

using namespace std;

std::string Music::current;
map<string,RMMUSICRef> Music::loaded;

void Music::setSilence(bool s) {
    silence=s;
};
bool Music::silence=false;
int Music::volume=0;

void Music::setVolume(int vol) {
    volume = vol;
    if (silence)
        return;
    Mix_VolumeMusic(vol);
};

int Music::getVolume() {
    if (silence)
        return volume;
    else
        return Mix_VolumeMusic(-1);
};

bool Music::isPlaying() {
    if (silence)
        return false;
    else
        return Mix_PlayingMusic();
};

void Music::whenFinished(void (*handler)()) {
    if (silence)
        return;
    Mix_HookMusicFinished(handler);
};


void Music::load(const std::string & name) {
    if (silence)
        return;
    if (loaded.find(name)==loaded.end()) {
        RMMUSICRef m(name);
        if (!&*m) {
            fprintf(stderr,"Unable to load music: %s\n",SDL_GetError());
            exit(1);
        }
        loaded[name]=m;
    }
};

void Music::unload(const std::string & name) {
    if (silence)
        return;
    if (current==name) 
        stop();
    loaded.erase(name);
};

void Music::unloadAll() {
    loaded.clear();
};

std::string Music::getCurrent() {
    return current;
}

int Music::play(const std::string & name,int loops) {
    if (silence) 
        return 0;
    stop();
    current=name;
    map<string,RMMUSICRef>::iterator m=loaded.find(name);
    if (m==loaded.end()) load(name);
    if (Mix_PlayMusic(&*loaded[name],loops)) {
        fprintf(stderr,"Unable to play music: %s\n",SDL_GetError());
        exit(1);
    } else return 0;
};

void Music::pause() {
    if (silence)
        return;
    Mix_PauseMusic();
};

void Music::resume() {
    if (silence)
        return;
    Mix_ResumeMusic();
};

void Music::stop() {
    if (silence)
        return;
    Mix_HaltMusic();
};
