/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef MUSIC_H
#define MUSIC_H

#include "resourcemanager.h"
#include <set>

/** Methods for playing music. */
class Music {
public:
    static void setVolume(int vol);
    static int getVolume();
    /** True if some music is being played. */
    static bool isPlaying();
    static void whenFinished(void (*handler)());
    /** 
     * Stops playing the current music if any and
     * plays a music with a given name, and loads it if it is not loaded.
     * */
    static int play(const std::string & name,int loops=0);
    static void stop();
    static void pause();
    static void resume();
    static std::string getCurrent();
    /** Loads a music resource. */
    static void load(const std::string & name);
    static void unload(const std::string & name);
    static void unloadAll();
    /** 
     * No more attention should be paid to calls to methods in this class is
     * silence is set to true.
     * */
    static void setSilence(bool);
private:
    Music() {};
    static bool silence;
    static int volume;
    static std::map<std::string,RMMUSICRef> loaded;
    static std::string current;
};

#endif //  SOUNDFX_H
