/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef PATH_H
#define PATH_H

#include <list>
#include "vector2d.h"
#include <SDL.h>
#include "global.h"

class Path {
public:
    static real addThreshold,deleteThreshold;

    template <class Iterator,class PopOp,class PushOp>
    bool addNode(Vector2d n,Iterator i,Iterator limit,PopOp pop,PushOp push);
    bool addNode(const Vector2d & n);
    bool addFrontNode(const Vector2d & n);
    bool deleteNode(Vector2d n);
    void draw(const Matrix2x3 & view,const Vector2d & pos,Uint32 color);
    void drawDashed(const Matrix2x3 & view,const Vector2d & pos,Uint32 color);
    void clear();
    Vector2d & front();
    void pop_front();
    Vector2d & back();
    void pop_back();

    int numNodes;
    Path();

    friend std::ostream & operator << (std::ostream& o,const Path& p);

    std::list<Vector2d> nodes;
private:
    Vector2d original;
    real distancebefore;
    int drawphase;
    int laststep;
};

#endif // PATH_H
