/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "parsercombinators.h"
#include <fstream>
#include <sstream>
#include <iostream>
#include <math.h>
#include <unistd.h>

#include "readconfig.h"
#include "debugmsg.h"

Config::Config(): audiobuffer(9),  musvolume(100), fxvolume(45), fullscreen(false) {};


std::ostream & operator << (std::ostream & o,const Config &c) {
    STDStreamPrinter sp(o);
    OStreamUTF8Encoder sc(&sp);

    sc<<"audiobuffer="<<c.audiobuffer<<'\n';
    sc<<"music volume="<<c.musvolume<<'\n';
    sc<<"fx volume="<<c.fxvolume<<'\n';
    sc<<"fullscreen="<<(c.fullscreen?"yes":"no")<<'\n';
    sc<<"width="<<c.width<<'\n';
    sc<<"height="<<c.height<<'\n';
    sc<<"language="<<c.language<<'\n';
    return o;
}

void readconfig(const char * file,Config * c) {
    if (!c) {
        CHERROR<<"Call readconfig with a valid value for it's second argument."<<ENDL;
        exit(1);
    }

    if (access(file,F_OK)) {
        CHERROR<<"Could not open file \""<<file<<'\"'<<ENDL;
        exit(1);
    }

    std::ifstream f(file);

    StreamTokenizer t(f);
    Lexer p=&t;
    PARSEbegin(Lexer,p);

    WS;
    LS("audiobuffer=");
    LINT(&c->audiobuffer);
    if (c->audiobuffer<8 || c->audiobuffer>13) {
        CHERROR<<"Error on file \""<<file<<"\": audiobuffer value is not between 8 and 13."<<ENDL;
        CHERROR<<"  Buffer size is suposed to be set at 2^audiobuffer.\n"<<ENDL;
        exit(1);
    }

    WS;
    LS("music volume=");
    LINT(&c->musvolume);
    if (c->musvolume<0 || c->musvolume>128) {
        CHERROR<<"Error on file \""<<file<<"\": music volume value is not between 0 and 128."<<ENDL;
        exit(1);
    }

    WS;
    LS("fx volume=");
    LINT(&c->fxvolume);
    if (c->fxvolume<0 || c->fxvolume>128) {
        CHERROR<<"Error on file \""<<file<<"\": fx volume value is not between 0 and 128."<<ENDL;
        exit(1);
    }

    WS;
    S("fullscreen=");
    OPTbegin;
    TRY(LS("yes"));
    c->fullscreen=true;
    OR
    LS("no");
    c->fullscreen=false;
    OPTend;

    WS;
    LS("width=");
    LINT(&c->width);

    WS;
    LS("height=");
    LINT(&c->height);

    WS;
    LS("language=");
    OPTbegin;
	    TRY(LS("en")); 
	    c->language="en"; 
	OR 
		TRY(LS("es"));
	    c->language="es"; 
    OPTend;

    WS;
    PEOF;
    PARSEend;

    if (p.error()) {
        CHERROR<<"Reading \""<<file<<"\": "<<p.errorMessage()<<ENDL;
        exit(1);
    }
}

