/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <guichan.hpp>
#include <guichan/sdl.hpp>
#include "graphic.h"
#include "screens.h"
#include "scorelist.h"
#include <sge.h>
#include "gcnlayouts.h"
#include "soundcollection.h"
#include "music.h"
#include "bar.h"
#include <sstream>
#include "parsercombinators.h"
#include "fade.h"
#include "debugmsg.h"
#include "clipping.h"
#include "imagescroller.h"

extern SoundCollection * sounds;

using namespace std;

extern int mousePositionX;
extern int mousePositionY;

const int initwait=1000;
int waitdelay=0;
int laststep;

/**
 * @returns false if the event loop must finish.
 */
bool GUI_process_events(gcn::SDLInput * input,gcn::SDLGraphics * graphics) {
    SDL_Event e;
    while (SDL_PollEvent(&e)) {
        if (waitdelay>0) continue;
        input->pushInput(e);
        switch (e.type) {
        case SDL_KEYDOWN:
            switch (e.key.keysym.sym) {
            case SDLK_ESCAPE:
                return false;
            default:
                ;
            }
            break;
        case SDL_MOUSEMOTION:
            mousePositionX=Graphic::fromScreenx(e.motion.x);
            mousePositionY=Graphic::fromScreeny(e.motion.y);
            break;
        case SDL_ACTIVEEVENT: {
            Uint8 st=SDL_GetAppState();
            if (st & (SDL_APPINPUTFOCUS | SDL_APPACTIVE)) {
                if (!e.active.gain)
                    Music::pause();
                else
                    Music::resume();
            }
            break;
            }
        case SDL_QUIT:
            throw 0;
            return false;
        default:
            ;
        }
    };
    return true;
};

class MainScreenAction : public gcn::ActionListener {
private:
    ButtonPress * st;
    bool * quit;
    int * levelcount;
    int leveltotal;
public:
    MainScreenAction(ButtonPress * st,bool * q,int * levelcount,int leveltotal) :
            st(st),quit(q),levelcount(levelcount),leveltotal(leveltotal) {};
    void  action (const gcn::ActionEvent& actionEvent) {
        const string id = actionEvent.getId();
        if (sounds && id!="quit" && id!="play" && id!="levelup" && id!="leveldown") sounds->mainScreenMenuButtons.play();
        if (id=="play") {
            if (sounds) sounds->playButton.play();
            *st=PLAYBT;
            *quit=true;
        } else if (id=="quit") {
            *quit=true;
        } else if (id=="scores") {
            *st=SCORESBT;
            *quit=true;
        } else if (id=="levelup") {
            if (leveltotal>0) {
                (*levelcount)++;
                *levelcount%=leveltotal;
            }
        } else if (id=="leveldown") {
            if (leveltotal>0) {
                (*levelcount)+=leveltotal-1;
                *levelcount%=leveltotal;
            }
        } else if (id=="story") {
            *st=STORYBT;
            *quit=true;
        } else if (id=="credits") {
            *st=CREDITSBT;
            *quit=true;
        } else if (id=="scores") {
            *st=SCORESBT;
            *quit=true;
        } else if (id=="options") {
            *st=OPTIONSBT;
            *quit=true;
        } else if (id=="editor") {
            *st=EDITORBT;
            *quit=true;
        } else if (id=="multiplayer") {
            *st=MULTIPLAYERBT;
            *quit=true;
        }
    }
};

extern int mousePositionX;
extern int mousePositionY;

class ImageButton : public gcn::Button {
public:
    ImageButton(const std::string & label,const std::string & imgid);

    void draw(gcn::Graphics *g) {
        int x=g->getCurrentClipArea().xOffset;
        int y=g->getCurrentClipArea().yOffset;
        img.setFrame(isPressed()?1:0);
        img.draw(x,y);
    }
private:
    Graphic img;
};

ImageButton::ImageButton(const std::string & label,const std::string & imgid) :
        gcn::Button(label), img(imgid,false) {
    setDimension(gcn::Rectangle(0,0,img.getWidth(),img.getHeight()));
    setFrameSize(0);
};


class FontButton : public gcn::Button {
public:
    FontButton(const std::string & label,const std::string & fontid,const std::string & fontgrayid);

    void draw(gcn::Graphics *g) {
        int x=g->getCurrentClipArea().xOffset;
        int y=g->getCurrentClipArea().yOffset;
        if (isPressed()) font->draw(x,y,getCaption());
        else fontgray->draw(x,y,getCaption());
    }

private:
    RMFONTRef font;
    RMFONTRef fontgray;
};

FontButton::FontButton(const std::string & label,const std::string & fontid,const std::string & fontgrayid) :
        gcn::Button(label), font(fontid), fontgray(fontgrayid) {
    setDimension(gcn::Rectangle(0,0,font->getWidth(label)+1,font->getHeight()));
    setFrameSize(0);
};


ButtonPress mainScreen(int & levelcount,int leveltotal) {

    SDL_WM_SetCaption("Spacejunk","Spacejunk");
    Graphic::setPalette("title",SDL_LOGPAL);

    gcn::SDLImageLoader imageLoader;
    gcn::Image::setImageLoader(&imageLoader);

    gcn::SDLGraphics graphics;
    graphics.setTarget(Graphic::getScreen());
    RMFONTRef font("asimovgray20");

    //ImageFont font("fixedfont.bmp", " abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789");
    gcn::Widget::setGlobalFont(&*font);

    gcn::SDLInput input;
    ButtonPress state=QUITBT;
    bool quit=false;

    MainScreenAction a(&state,&quit,&levelcount,leveltotal);
    //EditorControl ec;
    LeftVerticalLayout c;
    Graphic titlescreen("titlescreen",false,true);
    Graphic mousepointer("mousepointer",false);
    gcn::Label numlevel(getText("numlevelbt"));

    char s[5];
    snprintf(s,5,"%2d",levelcount+1);
    gcn::Label numl("00");
    numl.setCaption(s);
    FontButton ldown("<<","asimov20","asimovgray20"), lup(">>","asimov20","asimovgray20");
    CenterHorizontalLayout levell(5);
    ldown.setActionEventId("leveldown");
    ldown.addActionListener(&a);
    lup.setActionEventId("levelup");
    lup.addActionListener(&a);
    levell.add(&numlevel);
    levell.add(&ldown);
    levell.add(&numl);
    levell.add(&lup);
    levell.fitWidgets();

    FontButton bt(getText("playbt"),"asimov20","asimovgray20"),
    btStory(getText("storybt"),"asimov20","asimovgray20"),
    bt3(getText("scoresbt"),"asimov20","asimovgray20"),
    bt5(getText("optionsbt"),"asimov20","asimovgray20"),
    bt2(getText("quitbt"),"asimov20","asimovgray20"),
    bt7(getText("creditsbt"),"asimov20","asimovgray20");
    FontButton bt6(getText("leveleditorbt"),"asimov20","asimovgray20");
    bt.setActionEventId("play");
    bt.addActionListener(&a);
    bt3.setActionEventId("scores");
    bt3.addActionListener(&a);
    bt6.setActionEventId("editor");
    bt6.addActionListener(&a);
    bt5.setActionEventId("options");
    bt5.addActionListener(&a);
    bt7.setActionEventId("credits");
    bt7.addActionListener(&a);
    bt2.setActionEventId("quit");
    bt2.addActionListener(&a);
    btStory.setActionEventId("story");
    btStory.addActionListener(&a);
    c.add(&bt);
    c.add(&levell);
    c.add(&btStory);
    c.add(&bt5);
    c.add(&bt3);
    c.add(&bt6);
    c.add(&bt7);
    c.add(&bt2);
    c.setOpaque(false);
    c.fitWidgets();
    c.setPosition(Graphic::toScreenx(395),Graphic::toScreeny(148));
    gcn::Gui gui;
    gui.setGraphics(&graphics);
    gui.setInput(&input);
    gui.setTop(&c);

    int nx,ny;
    numl.getAbsolutePosition(nx,ny);
    SDL_Rect numlrect = {Graphic::fromScreenx(nx),Graphic::fromScreeny(ny),
                         int(numl.getWidth()/Graphic::getBaseScaleFactorX()),int(numl.getHeight()/Graphic::getBaseScaleFactorY())};
    Graphic::clearScreen(SDL_MapRGB(Graphic::getScreen()->format, 0, 0,0));
    Fade fade;
    fade.setMode(Fade::IN,1000);
    bool endfade=false;
    while (!quit && GUI_process_events(&input,&graphics)) {
        titlescreen.draw(0,0);
        gui.logic();
        gui.draw();
        if (!fade.step()) {
            endfade=true;
            break;
        }
        mousepointer.draw(mousePositionX,mousePositionY);

        Graphic::flip();
        Graphic::sleep(20);
    }
    if (endfade) {
        UPDATEbegin;
        bool first=true;
        while (!quit && GUI_process_events(&input,&graphics)) {
            UPDATEOFF(titlescreen.draw(0,0));
            gui.logic();

            char s[5];
            snprintf(s,5,"%2d",levelcount+1);
            numl.setCaption(s);
            Graphic::updateRect(&numlrect);

            gui.draw();
            mousepointer.draw(mousePositionX,mousePositionY);

            Graphic::flip(first);
            first=false;
            Graphic::sleep(20);
        }
        UPDATEend;
    }
    fade.setMode(Fade::OUT,state==PLAYBT?1000:500);
    while (fade.getMode()!=Fade::NONE) {
        GUI_process_events(&input,&graphics);
        titlescreen.draw(0,0);
        gui.logic();
        gui.draw();
        mousepointer.draw(mousePositionX,mousePositionY);
        if (!fade.step()) break;

        Graphic::flip();
        Graphic::sleep(20);
    }

    return state;
};


bool fallen_process_events(int * state) {
    SDL_Event e;
    while (SDL_PollEvent(&e)) {
        if (waitdelay>0) continue;
        switch (e.type) {
        case SDL_KEYDOWN:
            switch (e.key.keysym.sym) {
            case SDLK_ESCAPE:
                *state=0;
                return false;
            default:
                *state=1;
                return false;
            }
            break;
        case SDL_MOUSEBUTTONDOWN:
            *state=1;
            return false;
        case SDL_ACTIVEEVENT: {
            Uint8 st=SDL_GetAppState();
            if (st & (SDL_APPINPUTFOCUS | SDL_APPACTIVE)) {
                if (!e.active.gain)
                    Music::pause();
                else
                    Music::resume();
            }
            break;
            }
        case SDL_QUIT:
            throw 0;
            *state=0;
            return false;
        default:
            ;
        }
    };
    return true;
}

bool fallenScreen() {

    RMFONTRef fr("24p");
    Graphic planet("star3",false);
    int state=0;
    Graphic::clearScreen(SDL_MapRGB(Graphic::getScreen()->format, 0, 0,0));
    Graphic::flip();
    Graphic::clearScreen(SDL_MapRGB(Graphic::getScreen()->format, 0, 0,0));

    string text=getText("leveldied");
    laststep=Graphic::ticks();
    waitdelay=initwait;

    Fade f;
    f.setMode(Fade::IN,1000);
    while (fallen_process_events(&state)) {
        planet.draw(Graphic::screencenter-Vector2d(125,325));
        fr->drawL(int(Graphic::screencenter.x),int(Graphic::screencenter.y),text,FontRenderer::CENTER);
        f.step();
        Graphic::flip();
        Graphic::sleep(40);

        int delta=Graphic::ticks()-laststep;
        if (waitdelay<delta) waitdelay=0;
        else waitdelay-=delta;
        laststep+=delta;
    }

    f.setMode(Fade::OUT,1000);
    while (f.getMode()!=Fade::NONE) {
        planet.draw(Graphic::screencenter-Vector2d(125,325));
        fr->drawL(int(Graphic::screencenter.x),int(Graphic::screencenter.y),text,FontRenderer::CENTER);
        f.step();
        Graphic::flip();
        Graphic::sleep(40);
    }

    return state;
};


void lastLevelScreen() {

    Music::play("final");

    RMFONTRef fr("24p");
    Graphic ship("starship1",false);
    Graphic trash("trash8");
    Graphic trash1("trash6");
    Graphic trash2("trash2");
    Graphic g1("galaxia1",false);
    Graphic g2("galaxia0grande",false);
    Graphic g3("galaxia3",false);
    Graphic s1("farstargroup1",false);
    Graphic s2("farstargroup2",false);
    Graphic s3("farstargroup3",false);
    int state=0;
    Graphic::clearScreen(SDL_MapRGB(Graphic::getScreen()->format, 0, 0,0));
    Graphic::flip();
    Graphic::clearScreen(SDL_MapRGB(Graphic::getScreen()->format, 0, 0,0));
    string text=getText("endcongratulation");
    int init=laststep=Graphic::ticks();
    waitdelay=initwait;

    Fade f;
    f.setMode(Fade::IN,1000);
    while (fallen_process_events(&state)) {
        Graphic::clearScreen(SDL_MapRGB(Graphic::getScreen()->format, 0, 0,0));
        g1.draw(50,400);
        g2.draw(400,0);
        g3.draw(50,50);
        s1.draw(450,250);
        s2.draw(100,350);
        s2.draw(50,120);
        s3.draw(250,250);
        ship.draw(Graphic::screencenter-Vector2d(16,100+(laststep-init)*0.03));
        trash.draw(int(Graphic::screencenter.x),int(Graphic::screencenter.y+100),1,(laststep-init)*0.05);
        trash1.draw(100,50,1,(laststep-init)*0.1);
        trash2.draw(500,100,1,(laststep-init)*0.2);
        fr->drawL(int(Graphic::screencenter.x),int(Graphic::screencenter.y)-50,text,FontRenderer::CENTER);

        if (f.step()) break;
        Graphic::flip();
        Graphic::sleep(30);

        int delta=Graphic::ticks()-laststep;
        if (waitdelay<delta) waitdelay=0;
        else waitdelay-=delta;
        laststep+=delta;
    }

    UPDATEbegin;
    while (fallen_process_events(&state)) {
        Graphic::clearScreen(SDL_MapRGB(Graphic::getScreen()->format, 0, 0,0));
        UPDATEOFF(g1.draw(50,400));
        UPDATEOFF(g2.draw(400,0));
        UPDATEOFF(g3.draw(50,50));
        UPDATEOFF(s1.draw(450,250));
        UPDATEOFF(s2.draw(100,350));
        UPDATEOFF(s2.draw(50,120));
        UPDATEOFF(s3.draw(250,250));
        ship.draw(Graphic::screencenter-Vector2d(16,100+(laststep-init)*0.03));
        trash.draw(int(Graphic::screencenter.x),int(Graphic::screencenter.y+100),1,(laststep-init)*0.05);
        trash1.draw(100,50,1,(laststep-init)*0.1);
        trash2.draw(500,100,1,(laststep-init)*0.2);
        fr->drawL(int(Graphic::screencenter.x),int(Graphic::screencenter.y)-50,text,FontRenderer::CENTER);

        Graphic::flip();
        Graphic::sleep(30);

        int delta=Graphic::ticks()-laststep;
        if (waitdelay<delta) waitdelay=0;
        else waitdelay-=delta;
        laststep+=delta;
    }
    UPDATEend;

    f.setMode(Fade::OUT,500);
    while (f.getMode()!=Fade::NONE) {
        Graphic::clearScreen(SDL_MapRGB(Graphic::getScreen()->format, 0, 0,0));
        g1.draw(50,400);
        g2.draw(400,0);
        g3.draw(50,50);
        s1.draw(450,250);
        s2.draw(100,350);
        s2.draw(50,120);
        s3.draw(250,250);
        ship.draw(Graphic::screencenter-Vector2d(16,100+(laststep-init)*0.03));
        trash.draw(int(Graphic::screencenter.x),int(Graphic::screencenter.y)+100,1,(laststep-initwait)*0.05);
        trash1.draw(100,50,1,(laststep-init)*0.1);
        trash2.draw(500,100,1,(laststep-init)*0.2);
        fr->drawL(int(Graphic::screencenter.x),int(Graphic::screencenter.y)-50,text,FontRenderer::CENTER);
        f.step();
        Graphic::flip();
        Graphic::sleep(40);

        int delta=Graphic::ticks()-laststep;
        laststep+=delta;
    }
};

void scoreScreen(const ScoreList &scorelist) {
    RMFONTRef fr("12p");
    RMFONTRef fr2("asimov27");
    fr->linespacing=15;
    int state=0;
    Graphic background("scorescreen",false);
    Graphic::clearScreen(SDL_MapRGB(Graphic::getScreen()->format, 0, 0,0));
    string names=scorelist.getNames();
    string points=scorelist.getPoints();
    string title=getText("scoresbt");

    Fade f;
    f.setMode(Fade::IN,1000);
    while (fallen_process_events(&state)) {
        background.draw(0,0);
        fr->drawL(195,125,names);
        fr->drawL(450,125,points,FontRenderer::RIGHT);
        fr2->drawL((640-fr2->getWidth(title))/2,50,title);
        f.step();
        Graphic::flip();
        Graphic::sleep(40);
    }
    f.setMode(Fade::OUT,500);
    while (f.getMode()!=Fade::NONE) {
        background.draw(0,0);
        fr->drawL(195,125,names);
        fr->drawL(450,125,points,FontRenderer::RIGHT);
        fr2->drawL((640-fr2->getWidth(title))/2,50,title);
        f.step();
        Graphic::flip();
        Graphic::sleep(40);
    }
};

bool enter_score_process_events(sge_TextEditor & text) {
    SDL_Event e;
    while (SDL_PollEvent(&e)) {
        switch (e.type) {
        case SDL_KEYDOWN:
            switch (e.key.keysym.sym) {
            case SDLK_QUOTEDBL:
                break;
            case SDLK_RETURN:
                if (sounds) sounds->typeEnter.play();
            case SDLK_ESCAPE:
                if (text.get_string(false).length()>0 && sounds)
                    sounds->typeEnter.play();
                return false;
            case SDLK_DELETE:
            case SDLK_BACKSPACE:
            case SDLK_RSHIFT:
            case SDLK_LSHIFT:
            case SDLK_LEFT:
            case SDLK_RIGHT:
                text.check(&e);
                break;
            default:
                if (text.get_string(false).length()<30) {
                    if (sounds && e.key.keysym.unicode!=0) 
                        sounds->typeNotEnter.play();
                    text.check(&e);
                }
            }
            break;
        case SDL_ACTIVEEVENT: {
            Uint8 st=SDL_GetAppState();
            if (st & (SDL_APPINPUTFOCUS | SDL_APPACTIVE)) {
                if (!e.active.gain)
                    Music::pause();
                else
                    Music::resume();
            }
            break;
            }
        case SDL_QUIT:
            throw 0;
            return false;
        default:
            text.check(&e);
        }
    };
    return true;
}

string remapwsUint16(Uint16*str) {
    basic_string<wchar_t> s;
    for (;*str;str++) 
        s+=*str;
    return wstos(s);
}

void enterScoreScreen(ScoreList * scorelist,int playerPoints) {
    if (playerPoints<=0 || (!scorelist->inside(playerPoints) && scorelist->size()>=14)) return;

    Graphic background("scorescreen",false);
    RMFONTRef fr("12p");
    fr->linespacing=15;
    SDL_Rect r={0,0,LOGICAL_WIDTH,LOGICAL_HEIGHT};
    ScoreList::iterator i=scorelist->setScore("",playerPoints);
    int posy=scorelist->position(i)*(fr->getHeight()+fr->linespacing);
    while (scorelist->size()>14) {
        scorelist->erase_last();
    }

    string names=scorelist->getNames();
    string points=scorelist->getPoints();
    sge_TextEditor text;

    Uint16 * t;
    Fade f;
    f.setMode(Fade::IN,1000);
    while (enter_score_process_events(text)) {
        SDL_FillRect(Graphic::getScreen(),&r,SDL_MapRGB(Graphic::getScreen()->format, 0, 0,0));
        background.draw(0,0);
        fr->drawL(195,125,names);
        t=text.get_ucstring((Graphic::ticks()/300)%2);
        fr->drawLine(Graphic::toScreenx(195),Graphic::toScreeny(125)+posy,remapwsUint16(t));
        delete t;
        fr->drawL(450,125,points,FontRenderer::RIGHT);
        f.step();
        Graphic::flip();
        Graphic::sleep(40);
    }
    t=text.get_ucstring(false);
    if (*t!=L'\0') 
        scorelist->setName(remapwsUint16(t),i);
    else 
        scorelist->erase(i);
    delete t;

    f.setMode(Fade::OUT,500);
    while (f.getMode()!=Fade::NONE) {
        SDL_FillRect(Graphic::getScreen(),&r,SDL_MapRGB(Graphic::getScreen()->format, 0, 0,0));
        background.draw(0,0);
        fr->drawL(195,125,names);
        t=text.get_ucstring();
        fr->drawLine(Graphic::toScreenx(195),Graphic::toScreeny(125)+posy,remapwsUint16(t));
        delete t;
        fr->drawL(450,125,points,FontRenderer::RIGHT);
        f.step();
        Graphic::flip();
        Graphic::sleep(40);
    }
};


int countLines(const std::string& text) {
    int count=0;
    for(unsigned int i=0;i<text.length();i++) {
        if (text[i]=='\n')
            count++;
    }
    return count;
}


void storyScreen() {
    RMFONTRef fr("24p");
    RMFONTRef fr2("asimov27");
    int state=0;
    Graphic background("storyscreen",false);
    string story=getText("story");
    string title=getText("storybt");

    /*******************************************
     Get the text on a temporal surface
    *******************************************/
    SDL_Surface * screen=Graphic::getScreen();
    SDL_Surface * temp=Graphic::createSurface(Graphic::toScreenw(400),Graphic::toScreenh(520));
    SDL_FillRect(temp,NULL,SDL_MapRGB(screen->format, 0x00, 0x00, 0x00));
    fr->s=temp;
    fr->draw(0,0,story);
    fr->s=NULL;

    const int textheight = (fr->getHeight()+fr->linespacing)*countLines(story);
    const int windowheight = LOGICAL_HEIGHT-(150+50);
    const bool drawfull = textheight/Graphic::getBaseScaleFactorY()<=windowheight;
    SDL_Rect orig={0,-windowheight,400,windowheight};
    SDL_Rect dest={175,150,400,windowheight};
    ImageScroller imgs(temp,orig,0.010);


    /*******************************************
      Fade In
    *******************************************/
    
    Graphic::clearScreen(SDL_MapRGB(Graphic::getScreen()->format, 0, 0,0));
    Fade f;
    f.setMode(Fade::IN,1000);
    while (fallen_process_events(&state)) {
        Graphic::clearScreen(SDL_MapRGB(Graphic::getScreen()->format, 0, 0,0));
        background.draw(0,0);
        fr2->drawL(265,91,title);
        if (drawfull)
            imgs.draw(Graphic::getScreen(),175,dest.y,drawfull);
        if (!f.step()) 
            break;
        Graphic::flip();
        Graphic::sleep(30);
    }

    imgs.reset();
    bool first = true;
    UPDATEbegin
    while (fallen_process_events(&state)) {
        Graphic::clearScreen(SDL_MapRGB(Graphic::getScreen()->format, 0, 0,0));
        UPDATEOFF(background.draw(0,0);fr2->drawL(265,91,title));
        imgs.draw(Graphic::getScreen(),175,dest.y,drawfull);
        Graphic::updateRect(&dest);
        Graphic::flip(first);
        first = false;
        Graphic::sleep(30);
    }
    UPDATEend

    f.setMode(Fade::OUT,500);
    while (f.getMode()!=Fade::NONE) {
        Graphic::clearScreen(SDL_MapRGB(Graphic::getScreen()->format, 0, 0,0));
        background.draw(0,0);
        fr2->drawL(265,91,title);
        imgs.draw(Graphic::getScreen(),175,dest.y,drawfull);
        f.step();
        Graphic::flip();
        Graphic::sleep(30);
    }

    SDL_FreeSurface(temp);
};

void creditsScreen() {
    RMFONTRef fr("asimovgray20");
    RMFONTRef fr2("asimov27");
    int state=0;
    Graphic background("storyscreen",false);
    string credittitles=getText("credittitles");
    string creditnames=getText("creditnames");
    string title=getText("creditsbt");

    /*******************************************
     Get the text on a temporal surface
    *******************************************/
    SDL_Surface * screen=Graphic::getScreen();
    SDL_Surface * temp=Graphic::createSurface(Graphic::toScreenw(400),Graphic::toScreenh(520));
    SDL_FillRect(temp,NULL,SDL_MapRGB(screen->format, 0x00, 0x00, 0x00));
    fr->s=temp;
    fr->draw(Graphic::toScreenw(175),0,credittitles,FontRenderer::CENTER);
    fr->s=NULL;
    fr=RMFONTRef("asimov20");
    fr->s=temp;
    fr->draw(Graphic::toScreenw(175),0,creditnames,FontRenderer::CENTER);
    fr->s=NULL;

    /*******************************************
      Fade In
    *******************************************/

    Fade f;
    f.setMode(Fade::IN,1000);
    while (fallen_process_events(&state)) {
        Graphic::clearScreen(SDL_MapRGB(Graphic::getScreen()->format, 0, 0,0));
        background.draw(0,0);
        fr2->drawL(275,91,title);
        if (!f.step()) break;
        Graphic::flip();
        Graphic::sleep(30);
    }

    const int windowheight = LOGICAL_HEIGHT-(150+50);
    SDL_Rect orig={0,-windowheight,400,windowheight};
    SDL_Rect dest={150,150,400,windowheight};
    ImageScroller imgs(temp,orig,0.015);

    UPDATEbegin
    while (fallen_process_events(&state)) {
        Graphic::clearScreen(SDL_MapRGB(Graphic::getScreen()->format, 0, 0,0));
        UPDATEOFF(background.draw(0,0));
        UPDATEOFF(fr2->drawL(275,91,title));
        imgs.draw(Graphic::getScreen(),dest.x,dest.y,false);
        Graphic::updateRect(&dest);
        Graphic::flip(false);
        Graphic::sleep(30);
    }
    UPDATEend

    f.setMode(Fade::OUT,500);
    while (f.getMode()!=Fade::NONE) {
        Graphic::clearScreen(SDL_MapRGB(Graphic::getScreen()->format, 0, 0,0));
        background.draw(0,0);
        fr2->drawL(275,91,title);
        imgs.draw(Graphic::getScreen(),dest.x,dest.y,false);
        f.step();
        Graphic::flip();
        Graphic::sleep(30);
    }

    SDL_FreeSurface(temp);
};

bool giveUpScreen() {
    RMFONTRef fr("24p");
    Graphic planet("planet1",false);
    int state=0;
    Graphic::clearScreen(SDL_MapRGB(Graphic::getScreen()->format, 0, 0,0));
    Graphic::flip();
    Graphic::clearScreen(SDL_MapRGB(Graphic::getScreen()->format, 0, 0,0));
    string text=getText("giveup");

    Fade f;
    f.setMode(Fade::IN,1000);
    while (fallen_process_events(&state)) {
        planet.draw(Graphic::screencenter-Vector2d(50,150));
        fr->drawL(int(Graphic::screencenter.x),int(Graphic::screencenter.y),text,FontRenderer::CENTER);
        f.step();
        Graphic::flip();
        Graphic::sleep(40);
    }

    f.setMode(Fade::OUT,500);
    while (f.getMode()!=Fade::NONE) {
        planet.draw(Graphic::screencenter-Vector2d(50,150));
        fr->drawL(int(Graphic::screencenter.x),int(Graphic::screencenter.y),text,FontRenderer::CENTER);
        f.step();
        Graphic::flip();
        Graphic::sleep(40);
    }

    return state;
};

bool missionFailedScreen() {
    RMFONTRef fr("24p");
    Graphic planet("nebula1",false);
    int state=0;
    Graphic::clearScreen(SDL_MapRGB(Graphic::getScreen()->format, 0, 0,0));
    Graphic::flip();
    Graphic::clearScreen(SDL_MapRGB(Graphic::getScreen()->format, 0, 0,0));

    string text=getText("missionFailed");
    laststep=Graphic::ticks();
    waitdelay=initwait;

    Fade f;
    f.setMode(Fade::IN,1000);
    while (fallen_process_events(&state)) {
        planet.draw(Graphic::screencenter-Vector2d(75,175));
        fr->drawL(int(Graphic::screencenter.x),int(Graphic::screencenter.y),text,FontRenderer::CENTER);
        f.step();
        Graphic::flip();
        Graphic::sleep(40);

        int delta=Graphic::ticks()-laststep;
        if (waitdelay<delta) waitdelay=0;
        else waitdelay-=delta;
        laststep+=delta;
    }

    f.setMode(Fade::OUT,500);
    while (f.getMode()!=Fade::NONE) {
        planet.draw(Graphic::screencenter-Vector2d(75,175));
        fr->drawL(int(Graphic::screencenter.x),int(Graphic::screencenter.y),text,FontRenderer::CENTER);
        f.step();
        Graphic::flip();
        Graphic::sleep(40);
    }

    return state;
};



bool congratulation_process_events(int * state) {
    SDL_Event e;
    while (SDL_PollEvent(&e)) {
        if (waitdelay>0) continue;
        switch (e.type) {
        case SDL_KEYDOWN:
            switch (e.key.keysym.sym) {
            default:
                *state=1;
                return false;
            }
            break;
        case SDL_MOUSEBUTTONDOWN:
            *state=1;
            return false;
        case SDL_ACTIVEEVENT: {
            Uint8 st=SDL_GetAppState();
            if (st & (SDL_APPINPUTFOCUS | SDL_APPACTIVE)) {
                if (!e.active.gain)
                    Music::pause();
                else
                    Music::resume();
            }
            break;
        }
        case SDL_QUIT:
            throw 0;
            *state=0;
            return false;
        default:
            ;
        }
    };
    return true;
}


#define drawsumary Graphic::clearScreen(SDL_MapRGB(Graphic::getScreen()->format, 0, 0,0)); \
  UPDATEOFF(planet.draw(500,250));                  \
  UPDATEOFF(planet2.draw(200,200));                 \
  UPDATEOFF(farstar.draw(600,50));                  \
  UPDATEOFF(farstar.draw(350,150));                 \
  UPDATEOFF(farstar.draw(100,100));                 \
  UPDATEOFF(farstar.draw(300,430));                 \
  fuelbar.draw(units/TOTALFUELPOINTS);                  \
  UPDATEOFF(scoredisplay.draw((LOGICAL_WIDTH-scoredisplay.getWidth())/2,0)); \
  UPDATEOFF(shipcontrol.draw(0,LOGICAL_HEIGHT-shipcontrol.getHeight())); \
  o.str("");                                \
  o<<bonus;                             \
  fr->drawL(380,5,o.str(),FontRenderer::RIGHT);             \
  o.str("");                                \
  o<<*score;                                \
  fr->drawL(240,5,o.str(),FontRenderer::RIGHT);             \
  fr->drawL(520,5,"0",FontRenderer::RIGHT);             \
  fr2->drawL(int(Graphic::screencenter.x),int(Graphic::screencenter.y)-170,text,FontRenderer::CENTER); \
  UPDATEOFF(font24pgray->drawL(scorelabelx,1,scorelabel));   \
  UPDATEOFF(font24pgray->drawL(bonuslabelx,1,bonuslabel));  \
  UPDATEOFF(font24pgray->drawL(collectlabelx,1,collectlabel));

#define TOTALFUELPOINTS (50.0)
#define DISCOUNTUNIT (25)

bool sumaryScreen(real fuel,real maxFuel,int bonus,int * score) {

    Music::play("success");

    FuelBar fuelbar;
    RMFONTRef fr("12p");
    RMFONTRef fr2("24p");
    RMFONTRef font24pgray("24pgray");
    
    string scorelabel=getText("scorelabel");
    string bonuslabel=getText("bonuslabel");
    string collectlabel=getText("collectlabel");

    Graphic planet("star2",false);
    Graphic planet2("planet2",false);
    Graphic farstar("farstar1",false);

    Graphic scoredisplay("scoredisplay",false);
    Graphic shipcontrol("controlpanel",false);
    string text=getText("levelcongratulation");

    int scorex = (LOGICAL_WIDTH-scoredisplay.getWidth())/2;
    int scorelabelx = scorex + 59;
    int bonuslabelx = scorex + 200;
    int collectlabelx = scorex + 343;


    int units=int(floor(fuel*TOTALFUELPOINTS/maxFuel));
    int unitTime=0;
    int delta;
    int state=0;
    waitdelay=initwait;

    ostringstream o;

    Fade f;
    f.setMode(Fade::IN,1000);
    while (f.getMode()!=Fade::NONE) {
        drawsumary;
        f.step();
        Graphic::flip();
    }

    laststep=Graphic::ticks();
    delta=50;

    if (sounds) sounds->timeBack.play();

    while (congratulation_process_events(&state)) {
        drawsumary;
        Graphic::flip();

        if (units>0) {
            unitTime+=delta;
            if (unitTime>=DISCOUNTUNIT) {
                int addunits=unitTime/DISCOUNTUNIT;
                if (units<addunits) addunits=units;

                units-=addunits;
                *score+=addunits;
                unitTime%=DISCOUNTUNIT;
            }
        } else if (bonus>0) {
            unitTime+=delta;
            if (unitTime>=DISCOUNTUNIT) {
                int addunits=unitTime/DISCOUNTUNIT;
                if (bonus<addunits) addunits=bonus;

                bonus-=addunits;
                *score+=addunits;
                unitTime%=DISCOUNTUNIT;
            }
            if (bonus<=0 && sounds) sounds->timeBack.stop();
        }

        delta=Graphic::ticks()-laststep;
        if (delta<45) Graphic::sleep(45-delta);
        delta=Graphic::ticks()-laststep;
        if (waitdelay<delta) waitdelay=0;
        else waitdelay-=delta;
        laststep+=delta;
    }

    *score+=units;
    *score+=bonus;

    f.setMode(Fade::OUT,500);
    while (f.getMode()!=Fade::NONE) {
        drawsumary;
        f.step();
        Graphic::flip();
    }

    return state;
};


class ImageSlider : public gcn::Slider {
public:
    ImageSlider(float inf,float sup,const std::string & imgid);

    void setValue(double value) {
        if (value>getScaleEnd()) 
            gcn::Slider::setValue(getScaleEnd());
        else if (value<getScaleStart()) 
            gcn::Slider::setValue(getScaleStart());
        else 
            gcn::Slider::setValue(value);
    }

    void drawMarker(gcn::Graphics *g) {
        int x=g->getCurrentClipArea().xOffset;
        int y=g->getCurrentClipArea().yOffset;
        img.draw(int(1+Graphic::fromScreenx(int(x+getMarkerLength()/2+getValue()*(getWidth()-getMarkerLength()-1)/(getScaleEnd()-getScaleStart())))),Graphic::fromScreeny(y));
    }

    void draw(gcn::Graphics * g) {
        drawMarker(g);
    };

private:
    Graphic img;
};

ImageSlider::ImageSlider(float inf,float sup,const std::string & imgid) :
        gcn::Slider(inf,sup), img(imgid,false) {
    setMarkerLength(Graphic::toScreenw(img.getWidth()+10));
    setFrameSize(0);
    img.hotspot=Vector2d(img.getWidth()/2,0);
};


class SldListener : public gcn::MouseListener {
public:
    SldListener() {};
    void mouseReleased(gcn::MouseEvent& e) {
        if (sounds) 
            sounds->typeEnter.play();
    }
};

class OptionsAction : public gcn::ActionListener {
public:
    bool * quit;
    int * musvol, * fxvol;
    ImageSlider fxvolume,musicvolume;
    SldListener sldl;

    OptionsAction(bool * q, int * musvol, int * fxvol) :
            quit(q), musvol(musvol), fxvol(fxvol),
            fxvolume(0,100,"soundslidermarker"),
            musicvolume(0,128,"soundslidermarker") {
        fxvolume.setActionEventId("fxvolume");
        fxvolume.setValue(Soundfx::getOverallVolume());
        fxvolume.addActionListener(this);
        fxvolume.addMouseListener(&sldl);
        fxvolume.setDimension(gcn::Rectangle(Graphic::toScreenx(344),Graphic::toScreeny(237),
                                             Graphic::toScreenw(79),Graphic::toScreenh(16)));
        musicvolume.setActionEventId("musicvolume");
        musicvolume.setValue(Music::getVolume());
        musicvolume.addActionListener(this);
        musicvolume.setDimension(gcn::Rectangle(Graphic::toScreenx(344),Graphic::toScreeny(213),
                                                Graphic::toScreenw(79),Graphic::toScreenh(16)));
    };
    void  action (const gcn::ActionEvent& e) {
        const string id = e.getId();
        if (id=="done") {
            if (sounds) sounds->mainScreenMenuButtons.play();
            *quit=true;
        } else if (id=="musicvolume") {
            Music::setVolume(Uint8(musicvolume.getValue()));
            *musvol=Uint8(musicvolume.getValue());
        } else if (id=="fxvolume") {
            Soundfx::setOverallVolume(Uint8(fxvolume.getValue()));
            *fxvol=Uint8(fxvolume.getValue());
        }
    }
};

void optionsScreen(int * musvolume, int * fxvolume) {

    gcn::SDLImageLoader imageLoader;
    gcn::Image::setImageLoader(&imageLoader);

    gcn::SDLGraphics graphics;
    graphics.setTarget(Graphic::getScreen());
    RMFONTRef font("12p");
    RMFONTRef font24("24pgray");
    RMFONTRef fr2("asimov27");

    gcn::Widget::setGlobalFont(&*font);

    gcn::SDLInput input;
    bool quit=false;

    string title = getText("optionsbt");
    string musiclabel = getText("musiclabel");
    string fxlabel = getText("fxlabel");

    OptionsAction a(&quit,musvolume,fxvolume);

    Graphic background("soundscreen",false);
    Layout c;
    Graphic mousepointer("mousepointer",false);
    FontButton bt(getText("donebt"),"asimov20","asimovgray20");
    bt.setActionEventId("done");
    bt.addActionListener(&a);
    bt.setPosition(Graphic::toScreenx(285),Graphic::toScreeny(370));
    c.add(&a.musicvolume);
    c.add(&a.fxvolume);
    c.add(&bt);
    c.setOpaque(false);
    c.fitWidgets();
    gcn::Gui gui;
    gui.setGraphics(&graphics);
    gui.setInput(&input);
    gui.setTop(&c);

    Fade f;
    bool fullupdate=true;
    f.setMode(Fade::IN,1000);
    while (!quit && GUI_process_events(&input,&graphics)) {
        UPDATEOFF(Graphic::clearScreen(SDL_MapRGB(Graphic::getScreen()->format, 0, 0,0)));
        UPDATEOFF(background.draw(0,0));
        fr2->drawL(int(LOGICAL_WIDTH-fr2->getWidth(title)/Graphic::getBaseScaleFactorX())/2,147,title);
        font24->drawL(int(LOGICAL_WIDTH/2-10-font24->getWidth(musiclabel)/Graphic::getBaseScaleFactorX()),206,musiclabel);
        font24->drawL(int(LOGICAL_WIDTH/2-10-font24->getWidth(fxlabel)/Graphic::getBaseScaleFactorX()),229,fxlabel);
        gui.logic();
        gui.draw();
        f.step();
        mousepointer.draw(mousePositionX,mousePositionY);

        Graphic::flip(fullupdate);
        fullupdate = f.getMode()!=Fade::NONE;
        Graphic::sleep(20);
    }

    f.setMode(Fade::OUT,500);
    while (f.getMode()!=Fade::NONE) {
        UPDATEOFF(Graphic::clearScreen(SDL_MapRGB(Graphic::getScreen()->format, 0, 0,0)));
        UPDATEOFF(background.draw(0,0));
        fr2->drawL(int(LOGICAL_WIDTH-fr2->getWidth(title)/Graphic::getBaseScaleFactorX())/2,147,title);
        font24->drawL(int(LOGICAL_WIDTH/2-10-font24->getWidth(musiclabel)/Graphic::getBaseScaleFactorX()),206,musiclabel);
        font24->drawL(int(LOGICAL_WIDTH/2-10-font24->getWidth(fxlabel)/Graphic::getBaseScaleFactorX()),229,fxlabel);
        gui.draw();
        mousepointer.draw(mousePositionX,mousePositionY);
        f.step();
        Graphic::flip();
    }
};
