/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "serializer.h"
#include "bodyfactory.h"
#include "pathdrawer.h"
#include "starship.h"
#include <iomanip>
#include <iterator>
#include "functionexts.h"
#include <iostream>
#include <sstream>

using namespace std;


Lexer & parseWorld(Lexer & p,cWorld & w) {
    string type;
    basic_string<wchar_t> text;
    Vector2d pos,vel,eyeShift;
    double m,wangle,zoom;
    int flags,eyeid=0;
    GameBody * b;
    w.clear();
    int n=0;

    PARSEbegin(Lexer,p);
    LFLOAT(&zoom);
    w.zoom=zoom;
    LVECTOR(&eyeShift);
    LINT(&n);
    w.setBonusTime(n);

    // Read World description
    LATTTEXT(&text);
    w.description=wstos(text);
    // Read objects in the front world.
    w.getPlayerState()->mustCollect=0;

    LINT(&eyeid);
    LINT(&n);
    for (int i=0;i<n;i++) {
        basic_string<wchar_t> wtype;
        LID(&wtype);
        type=wstos(wtype);
        LVECTOR(&pos);
        LVECTOR(&vel);
        LFLOAT(&wangle);
        LFLOAT(&m);
        LINT(&flags);
        b=BodyFactory::instance()->create(type,pos,vel,m,wangle);
        b->flags=flags;
        w.addBody(b);
        if (type.find("trash")!=string::npos)
            w.getPlayerState()->mustCollect++;
        else if (type.find("starship")!=string::npos) {
            double fuel, maxFuel;
            LFLOAT(&fuel);
            LFLOAT(&maxFuel);
            static_cast<Starship*>(b)->setFuelNoMass(fuel);
            static_cast<Starship*>(b)->maxFuel=maxFuel;
            w.setShip(static_cast<Starship*>(b));
        }
    }
    ENDBLOCK;

    // Read objects in the background.
    LINT(&n);
    for (int i=0;i<n;i++) {
        basic_string<wchar_t> wtype;
        LID(&wtype);
        type=wstos(wtype);
        LVECTOR(&pos);
        LVECTOR(&vel);
        LFLOAT(&wangle)LFLOAT(&m);
        LINT(&flags);
        if (type.find("farstargroup")!=string::npos) {
            list<GameBody*> gbl=starlist(type,int(pos.x),int(pos.y),20);
            for (list<GameBody*>::iterator i=gbl.begin();i!=gbl.end();i++) {
                (*i)->flags=flags;
                w.bg.objects.push_front(*i);
            }
        } else {
            b=BodyFactory::instance()->create(type,pos,vel,m,wangle);
            b->flags=flags;
            w.bg.objects.push_front(b);
        }
    }
    ENDBLOCK;
    PARSEend;

    if (p.error()) {
        w.clear();
        return p;
    } else if (w.objectsBegin()!=w.objectsEnd()) {
        w.setTrackCoordinateSystem(w.getBody(eyeid));
        w.setEyeShift(eyeShift);
        if (w.pathDrawer && w.ship) w.pathDrawer->addFollow(w.ship->id);
    }

    return p;
}


ostream & Starship::serialize(ostream & o) const {
    return GameBody::serialize(o)<<" "<<fuel<<o.widen(' ')<<maxFuel;
};


OStreamUTF8Encoder & serializeGB(OStreamUTF8Encoder & sc,GameBody * g) {
    std::ostringstream o;
    g->serialize(o);
    sc<<o.str();
    return sc;
}

ostream & operator << (ostream & o,cWorld & w) {
    STDStreamPrinter sp(o);
    OStreamUTF8Encoder sc(&sp);
    sc<<w.getZoom()<<'\n';
    sc<<w.getEyeShift()<<'\n';
    sc<<w.getBonusTime()<<'\n';
    sc<<'"'<<stows(w.description)<<"\"\n";
    int total=0;
    int eyeid=0;
    for (cWorld::iterator i=w.objectsBegin();i!=w.objectsEnd();i++) {
        if (w.getTrackCoordinateSystem() && w.getTrackCoordinateSystem()->id==(*i)->id) eyeid=total;
        total++;
    }
    sc<<eyeid<<'\n';
    sc<<total<<'\n';
    for (cWorld::iterator i=w.objectsBegin();i!=w.objectsEnd();i++)
        serializeGB(sc,*i)<<'\n';
    total=0;
    for (std::list<GameBody*>::const_iterator i=w.bg.objects.begin();i!=w.bg.objects.end();i++)
        if (*i) total++;
    sc<<total<<'\n';
    for (std::list<GameBody*>::const_iterator i=w.bg.objects.begin();i!=w.bg.objects.end();i++)
        if (*i) serializeGB(sc,*i)<<'\n';

    return o;
}
