/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "soundfx.h"
#include "debugmsg.h"
#include "SDL_mixer.h"

#include <iostream>

using namespace std;


int Soundfx::nextChannel=0;

bool Soundfx::silence=false;
int Soundfx::volume=0;

void Soundfx::setSilence(bool s) {
    silence=s;
};
bool Soundfx::getSilence() {
    return silence;
}

void  Soundfx::setOverallVolume(int vol) {
	volume=vol;
	if (silence)
		return;
    Mix_Volume(-1,vol);
};

int Soundfx::getOverallVolume() {
	if (silence)
		return volume;
	else
        return Mix_Volume(-1,-1);
};


Soundfx::Soundfx(const std::string & name) : mc(name) {
    if (!&*mc) {
        fprintf(stderr,"Unable to load sound: %s\n",SDL_GetError());
        exit(1);
    }
};


int Soundfx::play(int loops) {
    if (silence) 
		return -1;

    int numc=Mix_AllocateChannels(-1);
    if (Mix_Playing(-1)==numc) {
        Mix_HaltChannel(nextChannel);
    }

    int c=Mix_PlayChannel(-1,&*mc,loops);
    if (c>=0) {
        nextChannel=(c+1)%numc;
        return c;
    } else {
        CHERROR<<"Error: Could not play sound."<<ENDL;
        return -1;
    }
};

void Soundfx::stop(int ch) {
    if (silence) 
		return;
    if (ch>=0) {
        if (Mix_GetChunk(ch)==&*mc) {
            Mix_HaltChannel(ch);
        }
    } else {
        for (int i=Mix_AllocateChannels(-1);i>=0;i--)
            if (Mix_GetChunk(i)==&*mc) Mix_HaltChannel(i);
    }
}

Soundfx::~Soundfx() {
};

