/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "zoomcontrol.h"


ZoomControl::ZoomControl(cWorld * w) : w(w),minus("zoomminus",false),plus("zoomplus",false), state(Z_NONE) {
};

#define BUTTONSEP 19

void ZoomControl::draw(int x,int y) {
    switch (state) {
    case Z_IN:
        plus.draw(x+BUTTONSEP,y);
        break;
    case Z_OUT:
        minus.draw(x,y);
        break;
    default:
        ;
    }
};

void ZoomControl::step(int) {};

void ZoomControl::pressIn() {
    state=Z_IN;
    w->zoomv=300.0/299.0;
};
void ZoomControl::pressOut() {
    state=Z_OUT;
    w->zoomv=299.0/300.0;
};
void ZoomControl::releaseIn() {
    if (state==Z_IN) release();
};
void ZoomControl::releaseOut() {
    if (state==Z_OUT) release();
};

void ZoomControl::release() {
    state=Z_NONE;
    w->zoomv=1;
}

void ZoomControl::mousePress(int x,int y,int mx,int my) {
    if (y<=my && my<=y+minus.getHeight() && x<=mx) {
        if (x+BUTTONSEP<=mx && mx<=x+BUTTONSEP+plus.getWidth())
            pressIn();
        else if (mx<x+minus.getWidth())
            pressOut();
    }
};


Button::Button(int x,int y,const std::string & releasedid,const std::string & pressedid) :
        released(releasedid,false), pressed(pressedid,false), btx(x), bty(y),
        btw(pressed.getWidth()), bth(pressed.getHeight()), state(RELEASED) {
};


bool Button::mousePressed(int x,int y,bool left) {
    if (btx<=x && x<=btx+btw && bty<=y && y<=bty+bth && pressed.insideImage(x-btx,y-bty)) {
        state=PRESSED;
        return true;
    }
    return false;
};

void Button::setState(STATE state) {
    this->state=state;
}

bool Button::mouseReleased(int x,int y,bool left) {
    if (state==RELEASED) return false;
    if (state==PRESSED) {
        state=RELEASED;
        return true;
    }
    state=RELEASED;
    return btx<=x && x<=btx+btw && bty<=y && y<=bty+bth && pressed.insideImage(x-btx,y-bty);
};

void Button::mouseMotion(int x,int y) {
    if (state==RELEASED) return;
    if (btx<=x && x<=btx+btw && bty<=y && y<=bty+bth && pressed.insideImage(x-btx,y-bty)) state=PRESSED;
    else state=ACTIVE;
};

void Button::draw() {
    if (state!=PRESSED) released.draw(btx,bty);
    else pressed.draw(btx,bty);
}

