/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <list>
#include "graphic.h"
#include <math.h>

#include "resourcemanager.h"
#include "sge.h"
#include "debugmsg.h"
#include <algorithm>
#include <iterator>
#include "parsercombinators.h"
#include "outline.h"
#include <sstream>
#include <fstream>
#include <iomanip>
#include "SDL_image.h"

using namespace std;

Point mousePosition;

int W=9;
int H=9;

void drawGrid(const Matrix2x3 & m,SDL_Surface * g){
  Uint32 graycolor=SDL_MapRGB(Graphic::getScreen()->format,128,128,128);
  for(int y=0;y<g->h;y++)
    for(int x=0;x<g->w;x++)
      if (SDLinsideImage(g,x,y)) {
	Vector2d xy=m*Vector2d(x,y);
	Vector2d xy1=m*Vector2d(x+1,y+1);
	Graphic::drawRect(int(xy.x),int(xy.y),int(xy1.x),int(xy1.y),graycolor);
      }
};

void drawVertexes(const Matrix2x3 & m,OUTLINE_Polygon* p){
  for(OUTLINE_Polygon::iterator i=p->begin();i!=p->end();i++){
    Vector2d p1=m**i;
    Vector2d p11=m*(*i+1);
    Graphic::drawFilledRect(int(p1.x),int(p1.y),int(p11.x),int(p11.y),
		   SDL_MapRGB(Graphic::getScreen()->format,255,255,255));
  }
  Vector2d p1=m*p->front();
    Vector2d p11=m*(p->front()+1);
    Graphic::drawFilledRect(int(p1.x),int(p1.y),int(p11.x),int(p11.y),
		   SDL_MapRGB(Graphic::getScreen()->format,0,255,0));
}

void drawLines(const Matrix2x3 & m,OUTLINE_Polygon* p,Uint32 color){
  OUTLINE_Polygon::iterator i=p->begin();
  Vector2d prev=*i;
  //  bool internal=i->internal;
  i++;
  for(;i!=p->end();i++){
    //    if (!internal) {
      Vector2d p0=m*(prev+0.5);
      Vector2d p1=m*(Vector2d(*i)+0.5);
      Graphic::drawArrow(int(p0.x),int(p0.y),int(p1.x),int(p1.y),color);
      //}
    prev=*i;
    //internal=i->internal;
  }
  //  if (!internal) {
  Vector2d p0=m*(prev+0.5);
  Vector2d p1=m*(Vector2d(p->front())+0.5);
  Graphic::drawArrow(int(p0.x),int(p0.y),int(p1.x),int(p1.y),color);
    //  }
}

void drawPolygons(const Matrix2x3 & m,const OUTLINE_Polygons & lp,Uint32 color){

  for(OUTLINE_Polygons::const_iterator p=lp.begin();p!=lp.end();p++){
      //drawVertexes(m,*p);

    drawLines(m,*p,color);

    list<Point> points;
    Circle c=minimumDisk((*p)->begin(),(*p)->end());
    Vector2d cxy=m*(c.center+0.5);
    Graphic::drawCircle(cxy,int(c.r*W),SDL_MapRGB(Graphic::getScreen()->format,0,255,0));
  }
};

string outname(const string &file){
  string::size_type p;
  if ((p=file.find_last_of('.'))!=string::npos){
    return file.substr(0,p)+".otl";
  } else return file+".otl";
}

bool process_events(Matrix2x3 & m,real & thresholdv,real & thresholda,OUTLINE_Polygons * p,char*file){
  SDL_Event e;
  while (SDL_PollEvent(&e)){
    switch(e.type){
    case SDL_KEYDOWN:        
      switch (e.key.keysym.sym){
      case SDLK_KP_PLUS:
      case SDLK_PLUS:
	thresholda=0.01;
	break;
      case SDLK_KP_MINUS:
      case SDLK_MINUS:
	thresholda=-0.01;
	break;
      case SDLK_s:
	{ofstream f(outname(file).c_str());
	  f<<(*p);
	  if (!f.fail()) CHERROR<<"outline saved to file "<<outname(file)<<ENDL;
	}
      case SDLK_ESCAPE: return false;
      default:;
      }break;
    case SDL_KEYUP:
      switch (e.key.keysym.sym){
      case SDLK_KP_PLUS:
      case SDLK_PLUS:
	thresholdv=thresholda=0;
	break;
      case SDLK_KP_MINUS:
      case SDLK_MINUS:
	thresholdv=thresholda=0;
	break;
      case SDLK_ESCAPE: return false;
      default:;
      }break;
    case SDL_MOUSEMOTION:
      mousePosition.x=e.motion.x;
      mousePosition.y=e.motion.y;
      break;
    case SDL_QUIT:
      return false;
    }
  }
  return true;
}


void test2(real threshold,char * file){
  SDL_Surface * g=IMG_Load(file);
  if (!g){
    CHERROR<<"Could not open file \""<<file<<"\": "<<SDL_GetError()<<ENDL;
    exit(1);
  }
  //  Graphic g(file,false);
  RMFONTRef f("24p");

  W=640/g->w;
  H=480/g->h;
  if (H<W) W=H;
  else H=W;

  real thresholdv=0;
  real thresholda=0;

  Matrix2x3 m(W,H,0,0,0,0,0);
  Uint32 whitecolor=SDL_MapRGB(Graphic::getScreen()->format,255,255,255);
  real lastnumber=-1;
  OUTLINE_Polygons * lp=OUTLINE_create(g,threshold);
  int sum=0;
  for(OUTLINE_Polygons::iterator p=lp->begin();p!=lp->end();p++) sum+=(*p)->size();

  UPDATEbegin;
  while(process_events(m,thresholdv,thresholda,lp,file)){
    thresholdv+=thresholda;
    if (threshold+thresholdv>=0) threshold+=thresholdv;
    Graphic::clearScreen(SDL_MapRGB(Graphic::getScreen()->format,0,0,0));
    drawGrid(m,g);
    drawPolygons(m,*lp,whitecolor);
    ostringstream s;
    s<<"number of vertexes: "<<sum;
    s<<"\nnumber of components: "<<lp->size();
    s<<"\npress +/- to change this threshold: "<<setprecision(2)<<fixed<<sqrt(threshold);
    f->draw(Graphic::getScreen()->w-10,10,s.str(),FontRenderer::RIGHT);
    if (lastnumber!=threshold){
      for(OUTLINE_Polygons::iterator p=lp->begin();p!=lp->end();p++) delete *p;
      delete lp;
      lp=OUTLINE_create(g,threshold);
      sum=0;
      for(OUTLINE_Polygons::iterator p=lp->begin();p!=lp->end();p++) sum+=(*p)->size();
      lastnumber=threshold;
    }

    SDL_Delay(20);
    Graphic::flip(false);
  }
  UPDATEend;
  for(OUTLINE_Polygons::iterator p=lp->begin();p!=lp->end();p++) delete *p;
  delete lp;
  SDL_FreeSurface(g);
}


void fillLoaderList(list<RLoader*> & rl){
  rl.push_back(Singleton<FontLoader>::getInstance());
}

int main(int argc,char**argv) {

  real threshold=0;
  if (argc>1) 
    threshold=strtod(argv[1],NULL);
  if (argc<3) {
    CHERROR<<"USAGE: outlinebuilder n filename"<<ENDL;
    exit(1);
  }

  //  CHERROR<<"Press +/- to round or sharpen the outline.\nPress s to save the outline to filename.otl, where filename is the one given on the command line."<<ENDL;

  if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE)!=0) {
    CHERROR<<SDL_GetError()<<ENDL;
    exit(1);
  }

  // Now we are ready to load resources
  list<RLoader*> rl;
  fillLoaderList(rl);
  readResources("data/resources.xml",rl);

  SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, 10);

  Graphic::set_video_mode(640,480);
  
  test2(threshold,argv[2]);

  SDL_Quit();
  return 0;
}

