/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <assert.h>
#include "collisionengine.h"
#include "cmask.h"
#include "resourcemanager.h"
#include <iostream>

using namespace std;

void fillLoaderList(list<RLoader*> & rl) {
    rl.push_back(Singleton<PaletteLoader>::getInstance());
    rl.push_back(Singleton<ImageSetLoader>::getInstance());
    rl.push_back(Singleton<CMaskLoader>::getInstance());
    rl.push_back(Singleton<MusicLoader>::getInstance());
    rl.push_back(Singleton<SoundLoader>::getInstance());
    rl.push_back(Singleton<FontLoader>::getInstance());
}


void TestCollision(){
  // load resources
  list<RLoader*> rl;
  fillLoaderList(rl);
  readResources("data/resources.xml",rl);

  if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE) != 0) {
    cerr<<"Unable to initialize SDL: "<<SDL_GetError()<<"\n";
    exit(1);
  }

  Graphic::set_video_mode(640,480);
  GameBody g1("planet1"),g2("planet1");
  Vector2d normal;
  real t;

  g1.setPosition(0,0);  g2.setPosition(0,0);
  g1.lastTestPos=Vector2d(0,0); g2.lastTestPos=Vector2d(0,0);
  assert(circle_swept_collision(&g1,&g2,&t,&normal));

  g1.setPosition(0,0);  g2.setPosition(20,0);
  g1.lastTestPos=Vector2d(0,0); g2.lastTestPos=Vector2d(-20,0);
  assert(!circle_swept_collision(&g1,&g2,&t,&normal));

  g1.setPosition(0,20);  g2.setPosition(20,0);
  g1.lastTestPos=Vector2d(0,-20); g2.lastTestPos=Vector2d(-20,0);
  assert(!circle_swept_collision(&g1,&g2,&t,&normal));

  g1.setPosition(0,60);  g2.setPosition(0,0);
  g1.lastTestPos=Vector2d(0,0); g2.lastTestPos=Vector2d(-60,0);
  assert(!circle_swept_collision(&g1,&g2,&t,&normal));

  g1.setPosition(0,40);  g2.setPosition(0,0);
  g1.lastTestPos=Vector2d(0,0); g2.lastTestPos=Vector2d(0,-40);
  assert(!circle_swept_collision(&g1,&g2,&t,&normal));

  g1.setPosition(0,40);  g2.setPosition(0,0);
  g1.lastTestPos=Vector2d(0,0); g2.lastTestPos=Vector2d(0,40);
  assert(circle_swept_collision(&g1,&g2,&t,&normal));

  SDL_Quit();
}
