/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <list>
#include "graphic.h"
#include <math.h>

#include "resourcemanager.h"
#include "sge.h"
#include "debugmsg.h"
#include <algorithm>
#include <iterator>
#include "parsercombinators.h"
#include "outline.h"
#include "SDL_image.h"
#include "cmask.h"
#include "clipping.h"
#include <fstream>

using namespace std;

Point mousePosition;

int W=9;
int H=9;

void drawVertexes(const Matrix2x3 & m,OUTLINE_Polygon* p){
  for(OUTLINE_Polygon::iterator i=p->begin();i!=p->end();i++){
    Vector2d p1=m**i;
    Vector2d p11=m*(*i+1);
    Graphic::drawFilledRect(int(p1.x),int(p1.y),int(p11.x),int(p11.y),
		   SDL_MapRGB(Graphic::getScreen()->format,255,255,255));
  }
  Vector2d p1=m*p->front();
    Vector2d p11=m*(p->front()+1);
    Graphic::drawFilledRect(int(p1.x),int(p1.y),int(p11.x),int(p11.y),
		   SDL_MapRGB(Graphic::getScreen()->format,0,255,0));
}

void drawLines(const Matrix2x3 & m,OUTLINE_Polygon* p,Uint32 color){
  OUTLINE_Polygon::iterator i=p->begin();
  Vector2d prev=*i;
  i++;
  for(;i!=p->end();i++){
      Vector2d p0=m*(prev+0.5);
      Vector2d p1=m*(Vector2d(*i)+0.5);
      Graphic::drawArrow(int(p0.x),int(p0.y),int(p1.x),int(p1.y),color);
      prev=*i;
  }
  Vector2d p0=m*(prev+0.5);
  Vector2d p1=m*(Vector2d(p->front())+0.5);
  Graphic::drawArrow(int(p0.x),int(p0.y),int(p1.x),int(p1.y),color);
}

void drawPolygons(const Matrix2x3 & m,const CMask & lp,Uint32 color){

  for(CMask::const_iterator p=lp.begin();p!=lp.end();p++){
    drawVertexes(m,p->first);
    
    drawLines(m,p->first,color);
    
    Circle c=minimumDisk(p->first->begin(),p->first->end());
    Vector2d cxy=m*(c.center+0.5);
    Graphic::drawCircle(cxy,int(c.r*W),SDL_MapRGB(Graphic::getScreen()->format,0,255,0));
  }
};


void drawInters(real angle,const Matrix2x3& m,Vector2d  bluelinep,Vector2d bluelined,
		const CMask & pol0,const CMask & pol1,const Vector2d &pos0,const Vector2d & pos1,
		SDL_Rect & r1,const Vector2d &vel){
  Point sumpos(int(pos0.x*W),int(pos0.y*H));
  Vector2d p0,p1;
  
  OUTLINE_PolygonReverser pr(Matrix2x3(1,1,angle,-r1.w/2,-r1.h/2,
				       pos1.x-pos0.x+r1.w/2,pos1.y-pos0.y+r1.h/2),pol1.front().first);
  OUTLINE_MKSum sum(pol0.front().first,-M_PI*angle/180,&pr);
  OUTLINE_Polygon * sumpol=sum.getPolygon();
  drawLines(Matrix2x3(1,1,0,0,0,sumpos.x,sumpos.y)*m,sumpol,SDL_MapRGB(Graphic::getScreen()->format,255,0,0));
  // draw origin
  p0=sumpos;
  p1=sumpos;
  p1+=W;
  Graphic::drawFilledRect(int(p0.x),int(p0.y),int(p1.x),int(p1.y),
			  SDL_MapRGB(Graphic::getScreen()->format,255,0,0));    

  real t0,t1;
  if (sum.intersect_line(0,vel,t0,t1)) {
    bluelinep=sumpos/W;
    bluelined=vel;
    p0=m*(bluelinep+t0*bluelined);
    Vector2d p01=m*(bluelinep+t0*bluelined+1);
    p1=m*(bluelinep+t1*bluelined);
    Vector2d p11=m*(bluelinep+t1*bluelined+1);
    Graphic::drawFilledRect(int(p0.x),int(p0.y),int(p01.x),int(p01.y),
			    SDL_MapRGB(Graphic::getScreen()->format,0,0,255));    

    Graphic::drawFilledRect(int(p1.x),int(p1.y),int(p11.x),int(p11.y),
			    SDL_MapRGB(Graphic::getScreen()->format,0,0,255));    
  }

  if (CMASK_intersect(&pol0,Matrix2x3(1,1,angle,-r1.w/2,-r1.h/2,pos1.x-pos0.x+r1.w/2,pos1.y-pos0.y+r1.h/2),M_PI*angle/180,
		      &pol1,vel)) {
    p0=sumpos;
    p1=sumpos;
    p1+=2*W;
    Graphic::drawFilledRect(int(p0.x),int(p0.y),int(p1.x),int(p1.y),
			    SDL_MapRGB(Graphic::getScreen()->format,0,0,255));    

  };
  p0=sumpos+W/2;
  p1=sumpos+vel*W;
  Graphic::drawArrow(int(p0.x),int(p0.y),int(p1.x),int(p1.y),SDL_MapRGB(Graphic::getScreen()->format,0,0,255));
  delete sumpol;
}


bool process_events(Matrix2x3 & m,Point & p,real & angle,Vector2d & vel){
  SDL_Event e;
  while (SDL_PollEvent(&e)){
    switch(e.type){
    case SDL_KEYDOWN:        
      switch (e.key.keysym.sym){
      case SDLK_PLUS:
      case SDLK_KP_PLUS:
	angle+=1;
	break;
      case SDLK_MINUS:
      case SDLK_KP_MINUS:
	angle-=1;
	break;
      case SDLK_LEFT:
	p.x-=1;
	break;
      case SDLK_RIGHT:
	p.x+=1;
	break;
      case SDLK_UP:
	p.y-=1;
	break;
      case SDLK_DOWN:
	p.y+=1;
	break;
      case SDLK_KP4:
	vel.x-=5;
	break;
      case SDLK_KP6:
	vel.x+=5;
	break;
      case SDLK_KP8:
	vel.y-=5;
	break;
      case SDLK_KP2:
	vel.y+=5;
	break;
      case SDLK_ESCAPE: return false;
      default:;
      }break;
    case SDL_MOUSEMOTION:
      mousePosition.x=e.motion.x;
      mousePosition.y=e.motion.y;
      break;
    }
  }
  return true;
}

void polygonBBOX(const OUTLINE_Polygon & poly,SDL_Rect * r){
  r->x=r->y=r->w=r->h=0;
  if (poly.size()==0) return;
  OUTLINE_Polygon::iterator i=poly.begin();
  r->x=i->x;r->y=i->y;

  for (;i!=poly.end();i++){
    if (i->x<r->x) {
      r->w+=r->x-i->x;
      r->x=i->x;
    }
    if (i->y<r->y) {
      r->h+=r->y-i->y;
      r->y=i->y;
    }
    if (r->x+r->w-1<i->x) r->w=i->x-r->x+1;
    if (r->y+r->h-1<i->y) r->h=i->y-r->y+1;
  }
};


void cmaskBBOX(const CMask & m,SDL_Rect * r){
  r->x=r->y=r->w=r->h=0;
  for(CMask::const_iterator i=m.begin();i!=m.end();i++){
    SDL_Rect temp;
    polygonBBOX(*i->first,&temp);
    addRect(r,temp);
  }
}

CMask * readMask(const char * c){
  ifstream f(c);
  if (f.fail()) {
    CHERROR<<"Could not open file \""<<c<<"\""<<ENDL;
    exit(1);
  }
  StreamTokenizer tok(f);
  Lexer p(&tok);
  CMask * m=new CMask();
  if (CMASK_parse(p,*m).error()) {
    CHERROR<<p.errorMessage()<<ENDL;
    exit(1);
  }
  return m;
}

void test2(const char * file0,const char * file1){
  CMask * m0=readMask(file0);
  if (!m0){
    CHERROR<<"Could not open \""<<file0<<ENDL;
    exit(1);
  }
  CMask * m1=readMask(file1);
  if (!m1){
    CHERROR<<"Could not open \""<<file1<<ENDL;
    exit(1);
  }

  SDL_Rect r0,r1;
  cmaskBBOX(*m0,&r0);
  cmaskBBOX(*m1,&r1);


  W=640/r0.w/4;
  H=480/r0.h/4;
  if (H<W) W=H;
  else H=W;


  Matrix2x3 m(W,H,0,0,0,0,0);
  Point pos1;
  Vector2d pos0(Graphic::getScreen()->w/4/W,Graphic::getScreen()->h/4/H);

  Point bluelined=Point(Graphic::getScreen()->w/4/W,Graphic::getScreen()->h/4/H);
  Vector2d bluelinep;
  real angle=0;
  Uint32 whitecolor=SDL_MapRGB(Graphic::getScreen()->format,255,255,255);
  Uint32 bluecolor=SDL_MapRGB(Graphic::getScreen()->format,0,0,255);
  Vector2d vel(30,0);

  UPDATEbegin;
  while(process_events(m,pos1,angle,vel)){
    bluelinep=Vector2d(real(mousePosition.x)/W,real(mousePosition.y)/H);
    Graphic::clearScreen(SDL_MapRGB(Graphic::getScreen()->format,0,0,0));

    drawPolygons(m*Matrix2x3(Vector2d(1,0),Vector2d(0,1),pos0),*m0,whitecolor);
    drawPolygons(m*Matrix2x3(1,1,angle,-r1.w/2,-r1.h/2,pos1.x+r1.w/2,pos1.y+r1.h/2),*m1,whitecolor);
    drawPolygons(m*Matrix2x3(1,1,angle,-r1.w/2,-r1.h/2,int(pos1.x+vel.x+r1.w/2),int(pos1.y+vel.y+r1.h/2)),*m1,bluecolor);

    drawInters(angle,m,bluelinep,bluelined,*m0,*m1,pos0,pos1,r1,vel);

    SDL_Delay(20);
    Graphic::flip(false);
  }
  UPDATEend;

}


int main(int argc,char**argv) {

  if (argc!=3) {
    CHERROR<<"USAGE: testoutline filename filename"<<ENDL;
    exit(1);
  }

  if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE)!=0) {
    CHERROR<<SDL_GetError()<<ENDL;
    exit(1);
  }


  SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, 10);

  Graphic::set_video_mode(640,480);
  
  test2(argv[1],argv[2]);

  SDL_Quit();
  return 0;
}
