/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <list>
#include "graphic.h"
#include <math.h>

#include "resourcemanager.h"
#include "sge.h"
#include "debugmsg.h"
#include <algorithm>
#include <iterator>
#include <assert.h>
#include "parsercombinators.h"
#include "outline.h"
#include "SDL_image.h"


using namespace std;

Point mousePosition;

int W=9;
int H=9;

void drawGrid(const Matrix2x3 & m,SDL_Surface * g){
  Uint32 graycolor=SDL_MapRGB(Graphic::getScreen()->format,128,128,128);
  for(int y=0;y<g->h;y++)
    for(int x=0;x<g->w;x++)
      if (SDLinsideImage(g,x,y)) {
	Vector2d xy=m*Vector2d(x,y);
	Vector2d xy1=m*Vector2d(x+1,y+1);
	Graphic::drawRect(int(xy.x),int(xy.y),int(xy1.x),int(xy1.y),graycolor);
      }
};

void drawVertexes(const Matrix2x3 & m,OUTLINE_Polygon* p){
  for(OUTLINE_Polygon::iterator i=p->begin();i!=p->end();i++){
    Vector2d p1=m**i;
    Vector2d p11=m*(*i+1);
    Graphic::drawFilledRect(int(p1.x),int(p1.y),int(p11.x),int(p11.y),
		   SDL_MapRGB(Graphic::getScreen()->format,255,255,255));
  }
  Vector2d p1=m*p->front();
    Vector2d p11=m*(p->front()+1);
    Graphic::drawFilledRect(int(p1.x),int(p1.y),int(p11.x),int(p11.y),
		   SDL_MapRGB(Graphic::getScreen()->format,0,255,0));
}

void drawLines(const Matrix2x3 & m,OUTLINE_Polygon* p,Uint32 color){
  OUTLINE_Polygon::iterator i=p->begin();
  Vector2d prev=*i;
  //  bool internal=i->internal;
  i++;
  for(;i!=p->end();i++){
    //    if (!internal) {
      Vector2d p0=m*(prev+0.5);
      Vector2d p1=m*(Vector2d(*i)+0.5);
      Graphic::drawArrow(int(p0.x),int(p0.y),int(p1.x),int(p1.y),color);
      //}
    prev=*i;
    //internal=i->internal;
  }
  //  if (!internal) {
  Vector2d p0=m*(prev+0.5);
  Vector2d p1=m*(Vector2d(p->front())+0.5);
  Graphic::drawArrow(int(p0.x),int(p0.y),int(p1.x),int(p1.y),color);
    //  }
}

void drawPolygons(const Matrix2x3 & m,const OUTLINE_Polygons & lp,Uint32 color){

    for(OUTLINE_Polygons::const_iterator p=lp.begin();p!=lp.end();p++){
      drawVertexes(m,*p);

      drawLines(m,*p,color);

      list<Point> points;
      for(OUTLINE_Polygon::iterator i=(*p)->begin();i!=(*p)->end();i++) points.push_back(*i);
      Circle c=minimumDisk(points.begin(),points.end());
      Vector2d cxy=m*(c.center+0.5);
      Graphic::drawCircle(cxy,int(c.r*W),SDL_MapRGB(Graphic::getScreen()->format,0,255,0));
    }
};


void drawInters(real angle,const Matrix2x3& m,const Vector2d & bluelinep,const Point & bluelined,
		const OUTLINE_Polygon * pol0,const OUTLINE_Polygon * pol1,const Vector2d &pos0,const Vector2d & pos1,
		SDL_Surface * g1){
  Point sumpos(int(pos0.x*W),int(pos0.y*H));
  Vector2d p0,p1;
  real t0,t1;
  
  OUTLINE_PolygonReverser pr(Matrix2x3(1,1,angle,-g1->w/2,-g1->h/2,
				       pos1.x-pos0.x+g1->w/2,pos1.y-pos0.y+g1->h/2),pol1);
  OUTLINE_MKSum sum(pol0,M_PI*angle/180,&pr);
  OUTLINE_Polygon * sumpol=sum.getPolygon();
  OUTLINE_Polygon * reversedpol=pr.getPolygon();
  drawLines(Matrix2x3(1,1,0,0,0,sumpos.x,sumpos.y)*m,sumpol,SDL_MapRGB(Graphic::getScreen()->format,255,0,0));
  drawVertexes(Matrix2x3(1,1,0,0,0,sumpos.x,sumpos.y)*m,sumpol);
  
  drawLines(Matrix2x3(1,1,0,0,0,sumpos.x,sumpos.y)*m,reversedpol,SDL_MapRGB(Graphic::getScreen()->format,0,0,255));

  // draw origin
  p0=sumpos;
  p1=sumpos;
  p1+=W;
  Graphic::drawFilledRect(int(p0.x),int(p0.y),int(p1.x),int(p1.y),
			  SDL_MapRGB(Graphic::getScreen()->format,255,0,0));    

  if (sum.intersect_line(bluelinep-sumpos/W,bluelined,t0,t1)) {
    p0=m*(bluelinep+t0*bluelined);
    Vector2d p01=m*(bluelinep+t0*bluelined+1);
    p1=m*(bluelinep+t1*bluelined);
    Vector2d p11=m*(bluelinep+t1*bluelined+1);
    Graphic::drawFilledRect(int(p0.x),int(p0.y),int(p01.x),int(p01.y),
			    SDL_MapRGB(Graphic::getScreen()->format,0,0,255));    

    Graphic::drawFilledRect(int(p1.x),int(p1.y),int(p11.x),int(p11.y),
			    SDL_MapRGB(Graphic::getScreen()->format,0,0,255));    

    if (sum.inside((mousePosition-sumpos+1)/W)){
      p0=mousePosition;
      p1=mousePosition+W;
      Graphic::drawCircle(p0,5,SDL_MapRGB(Graphic::getScreen()->format,255,0,0));    
    }
  };

  OUTLINE_MKSum::Index e;
  if (sum.closest_inside((mousePosition-sumpos)/W,e)) {
      Graphic::drawCircle(mousePosition,10*W,SDL_MapRGB(Graphic::getScreen()->format,255,255,255));
  } else {
    Vector2d ve=sum[sum.next(e)]-sum[e];
    real sin=dot(ve.rot90(),(mousePosition-sumpos)/W-sum[e]);
    if (sin*sin/ve.sqrDist()<=100)
      Graphic::drawCircle(mousePosition,10*W,SDL_MapRGB(Graphic::getScreen()->format,255,255,255));
  }

  p0=m*(bluelinep+0.5);
  p1=m*(bluelinep+bluelined);
  Graphic::drawArrow(int(p0.x),int(p0.y),int(p1.x),int(p1.y),SDL_MapRGB(Graphic::getScreen()->format,0,0,255));
  delete sumpol;
  delete reversedpol;
}


bool process_events(Matrix2x3 & m,Point & p,real & angle){
  SDL_Event e;
  while (SDL_PollEvent(&e)){
    switch(e.type){
    case SDL_KEYDOWN:        
      switch (e.key.keysym.sym){
      case SDLK_PLUS:
      case SDLK_KP_PLUS:
	angle+=1;
	break;
      case SDLK_MINUS:
      case SDLK_KP_MINUS:
	angle-=1;
	break;
      case SDLK_LEFT:
	p.x-=1;
	break;
      case SDLK_RIGHT:
	p.x+=1;
	break;
      case SDLK_UP:
	p.y-=1;
	break;
      case SDLK_DOWN:
	p.y+=1;
	break;
      case SDLK_ESCAPE: return false;
      default:;
      }break;
    case SDL_MOUSEMOTION:
      mousePosition.x=e.motion.x;
      mousePosition.y=e.motion.y;
      break;
    }
  }
  return true;
}

void test2(real threshold,char * file0,char * file1){
  SDL_Surface * g0=IMG_Load(file0);
  if (!g0){
    CHERROR<<"Could not open file \""<<file0<<"\": "<<SDL_GetError()<<ENDL;
    exit(1);
  }
  SDL_Surface * g1=IMG_Load(file1);
  if (!g1){
    CHERROR<<"Could not open file \""<<file1<<"\": "<<SDL_GetError()<<ENDL;
    exit(1);
  }


  W=640/max(g0->w,g1->w)/4;
  H=480/max(g0->h,g1->h)/4;
  if (H<W) W=H;
  else H=W;

  OUTLINE_Polygon tpl(20);
  for(int i=0;i<tpl.size();i++) 
	  assert(!tpl.isInternal(i));
  for(int i=1;i<tpl.size()-1;i+=2) {
    tpl.setInternal(i,true);
    assert(tpl.isInternal(i));
    assert(!tpl.isInternal(i-1));
    assert(!tpl.isInternal(i+1));
  }

  real t0,t1;
  assert(line_line_intersect(Vector2d(0,0),Vector2d(1,0),Vector2d(0,-1),Vector2d(0,1),t0,t1));
  assert(t0==0);
  assert(t1==1);
  assert(line_line_intersect(Vector2d(-0.5,0),Vector2d(1,0),Vector2d(0,-1.5),Vector2d(0,1),t0,t1));
  assert(t0==0.5);
  assert(t1==1.5);

  OUTLINE_Polygons * lp0=OUTLINE_create(g0,threshold);;
  OUTLINE_Polygons * lp1=OUTLINE_create(g1,threshold);

  Matrix2x3 m(W,H,0,0,0,0,0);
  Point pos1;
  Vector2d pos0((Graphic::getScreen()->w-g0->w)/2/W,(Graphic::getScreen()->h-g0->h)/2/H);

  Point bluelined=Point(Graphic::getScreen()->w/4/W,Graphic::getScreen()->h/4/H);
  Vector2d bluelinep;
  real angle=0;
  Uint32 whitecolor=SDL_MapRGB(Graphic::getScreen()->format,255,255,255);

  UPDATEbegin;
  while(process_events(m,pos1,angle)){
    bluelinep=Vector2d(real(mousePosition.x)/W,real(mousePosition.y)/H);
    Graphic::clearScreen(SDL_MapRGB(Graphic::getScreen()->format,0,0,0));
    drawGrid(m*Matrix2x3(Vector2d(1,0),Vector2d(0,1),pos0),g0);
    drawGrid(m*Matrix2x3(1,1,angle,-g1->w/2,-g1->h/2,pos1.x+g1->w/2,pos1.y+g1->h/2),g1);

    drawPolygons(m*Matrix2x3(Vector2d(1,0),Vector2d(0,1),pos0),*lp0,whitecolor);
    drawPolygons(m*Matrix2x3(1,1,angle,-g1->w/2,-g1->h/2,pos1.x+g1->w/2,pos1.y+g1->h/2),*lp1,whitecolor);

    drawInters(angle,m,bluelinep,bluelined,lp0->front(),lp1->front(),pos0,pos1,g1);

    SDL_Delay(20);
    Graphic::flip(false);
  }
  UPDATEend;

  printf("number of components lp0: %d\n",lp0->size());
  for(OUTLINE_Polygons::iterator p=lp0->begin();p!=lp0->end();p++) delete *p;

  printf("number of components lp1: %d\n",lp1->size());
  for(OUTLINE_Polygons::iterator p=lp1->begin();p!=lp1->end();p++) delete *p;

  delete lp0;
  delete lp1;
  SDL_FreeSurface(g0);
  SDL_FreeSurface(g1);
}

int main(int argc,char**argv) {

  real threshold=0;
  if (argc>1) 
    threshold=strtod(argv[1],NULL);
  if (argc<4) {
    CHERROR<<"USAGE: testoutline n filename filename"<<ENDL;
    exit(1);
  }

  if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE)!=0) {
    CHERROR<<SDL_GetError()<<ENDL;
    exit(1);
  }

  SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, 10);

  Graphic::set_video_mode(640,480);
  
  test2(threshold,argv[2],argv[3]);

  SDL_Quit();
  return 0;
}
















/*bool checkThreshold(const Point & prev,const Point & next,Poly & nodes,real threshold){

  // look at area
  nodes.push_front(prev);
  nodes.push_front(next);
  real area=polygonArea(nodes);
  nodes.pop_front();
  nodes.pop_front();
  Vector2d d1=next-nodes.back();
  Vector2d d2=nodes.back()-prev;
  //  CHERROR<<area<<" "<<Vector2d(prev)<<" "<<Vector2d(nodes.back())<<" "<<Vector2d(next)<<ENDL;
  return (dot(d1,d2)>=0 || dot(d1.rot90(),d2)!=0) && area<threshold;
}

bool checkThresholdAngle(const Point & prev,const Point & next,Poly & nodes,real threshold){
  Vector2d v=next-prev;
  if (v==0) return false;
  // look at angles
  for(Poly::const_iterator i=nodes.begin();i!=nodes.end();i++){
    Point oldsegment=*i-prev;
    real d=dot(v.rot90(),oldsegment);
    if (d*d>=threshold*v.sqrDist()){
      return false;
    }
  }
  return true;
}

void optimizePolygonLines(Poly & poly,real threshold){
  if (poly.empty()) return;
  Poly nodes;

  Poly::iterator i=poly.begin();
  Point prev=*i++;
  if (i!=poly.end()){
    Poly::iterator orig=i++;
    if (i!=poly.end()){
      poly.push_back(prev);
      
      do {
	nodes.push_back(*orig);
	if (checkThreshold(prev,*i,nodes,threshold))
	    poly.erase(orig);
	else {
	  prev=*orig;
	  nodes.clear();
	}
	orig=i;
	i++;
      } while (i!=poly.end());

      poly.pop_back();
      i=poly.begin();
      i++;
      nodes.push_back(poly.front());
      if (checkThreshold(prev,*i,nodes,threshold))
	poly.pop_front();
    }
  }
};

void optimizePolygonLineAngles(Poly & poly,real threshold){
  if (poly.empty()) return;
  Poly nodes;

  Poly::iterator i=poly.begin();
  Point prev=*i++;
  if (i!=poly.end()){
    Poly::iterator orig=i++;
    if (i!=poly.end()){
      poly.push_back(prev);
      
      do {
	nodes.push_back(*orig);
	if (checkThresholdAngle(prev,*i,nodes,threshold))
	    poly.erase(orig);
	else {
	  prev=*orig;
	  nodes.clear();
	}
	orig=i;
	i++;
      } while (i!=poly.end());

      poly.pop_back();
      i=poly.begin();
      i++;
      nodes.push_back(poly.front());
      if (checkThresholdAngle(prev,*i,nodes,threshold))
	poly.pop_front();
    }
  }
};


real messureB(Poly::iterator i,Poly & nodes){
  Point prev=i!=nodes.begin() ? (*(--i)++) : nodes.back();
  Point next=++i!=nodes.end() ? (*i) : nodes.front();
  i--;

  Poly mynodes;
  
  // look at area
  mynodes.push_front(prev);
  mynodes.push_front(next);
  mynodes.push_front(*i);
  return fabs(polygonArea(mynodes));
}



bool segment_line_intersect(Point p0,Point p1,Point q0,Point q1){
  // (q1.y-q0.y)(q.x-q0.x)-(q1.x-q0.x)(q.y-q0.y)=0
  return ((q1.y-q0.y)*(p0.x-q0.x)-(q1.x-q0.x)*(p0.y-q0.y))*((q1.y-q0.y)*(p1.x-q0.x)-(q1.x-q0.x)*(p1.y-q0.y))<0;
}


template <class List>
typename List::iterator nextit(typename List::iterator i,List & p){
  if (++i==p.end()) return p.begin();
  else return i;
};

template <class List>
typename List::iterator previt(typename List::iterator i,List & p){
  if (i==p.begin()) return --p.end();
  else return --i;
};

void polygon_intersect(list<Poly2::iterator> & ps,Poly2 & nodes,Poly2::iterator begin,Poly2::iterator end,Point p0,Point p1){
  if (begin==end) return;
  Poly2::iterator i=begin;
  Poly2::iterator prev;
  Vector2d n;
  do {
    prev=i;
    i=nextit(i,nodes);
    if (segment_segment_intersect(*prev,*i,p0,p1,n)) ps.push_front(prev);
  } while (i!=end);
}


class Circle { 
 public:
  real r;
  Vector2d center;
  Circle() { r=0; }
  Circle(Vector2d center,real r) : r(r), center(center) {}
  template <class T>
  bool contains(const vector2d<T> & p) const {
    return (center-p).sqrDist()<=r*r;
  };
  real area() const { return r*r*M_PI; }
};

Circle find_circle(const list<Point> & border){
  list<Point>::const_iterator first=border.begin();
  if (first==border.end()) return Circle();
  list<Point>::const_iterator second=first;
  second++;
  if (second==border.end()) return Circle(*first,0);
  list<Point>::const_iterator third=second;
  third++;
  Vector2d a0=Vector2d(*second-*first)/2.0;
  if (third==border.end()) return Circle(a0+*first,a0.dist());

  Vector2d a1=Vector2d(*third-*second)/2.0;
  Vector2d n;
  
  if ((a1+*second-*first).sqrDist()<=a1.sqrDist())
    return Circle(a1+*second,a1.dist());
  else if ((a0+*first-*third).sqrDist()<=a0.sqrDist())
    return Circle(a0+*first,a0.dist());
  else if ((Vector2d(*first+*third)/2.0-*second).sqrDist()<=(*first-*third).sqrDist()/4.0)
    return Circle(Vector2d(*first+*third)/2.0,(*first-*third).dist()/2.0);

  if (line_line_intersect(a1+*second,a1.rot90(),a0+*first,a0.rot90(),n))
    return Circle(n,(n-*first).dist());
  else {
    CHERROR<<"find_circle: Something impossible happened: "<<*first<<" "<<*second<<" "<<*third<<ENDL;
    exit(1);
  }
}

Circle minimumDiskWithBorder(list<Point>::iterator begin,list<Point>::iterator end,list<Point> & border){
  if (begin==end || border.size()==3)
    return find_circle(border);
  else {
    list<Point>::iterator second=begin;
    second++;
    Circle c=minimumDiskWithBorder(second,end,border);
    if (c.contains(*begin)) return c;
    
    border.push_front(*begin);
    c=minimumDiskWithBorder(second,end,border);
    border.pop_front();
    return c;
  }
}

Circle minimumDisk(list<Point>::iterator begin,list<Point>::iterator end){
  list<Point> border;
  return minimumDiskWithBorder(begin,end,border);
};

real circle_intersection_area(const Circle & c0,const Circle & c1){
  // Test if one circumference surrounds the other.
  real r=fabs(c1.r-c0.r);
  if ((c1.center-c0.center).sqrDist()<=r*r) 
    if (c1.r>c0.r) return c0.area();
    else return c1.area();

  // Test if they intersect at all
  r=c0.r+c1.r;
  if (r*r<=(c1.center-c0.center).sqrDist()) return 0;
  
  // Intersections
  real cdist=(c1.center-c0.center).dist();
  real xi0=(c1.r*c1.r-c0.r*c0.r-cdist*cdist)/(2*cdist);
  real xi1=(c1.r*c1.r-c0.r*c0.r+cdist*cdist)/(2*cdist);

  real ai0=c0.r*c0.r*(M_PI-asin(fabs(xi0)/c0.r))+fabs(xi0*sqrt(c0.r*c0.r-xi0*xi0));
  real ai1=c1.r*c1.r*(M_PI-asin(fabs(xi1)/c1.r))+fabs(xi1*sqrt(c1.r*c1.r-xi1*xi1));

  if (xi0*xi1>=0)
    if (c1.r>c0.r) return ai0+c1.area()-ai1;
    else return ai1+c0.area()-ai0;
  else return c0.area()-ai0+c1.area()-ai1;
}

real messureDivision(Poly2 & nodes,Poly2::iterator pos,Poly2::iterator current){
  list<Point> component;
  for(Poly2::iterator i=pos;i!=current;i=nextit(i,nodes))
    component.push_back(*i);
  component.push_back(*current);
  Circle c0=minimumDisk(component.begin(),component.end());

  component.clear();
  for(Poly2::iterator i=current;i!=pos;i=nextit(i,nodes))
    component.push_back(*i);
  component.push_back(*pos);
  Circle c1=minimumDisk(component.begin(),component.end());

  real a=c0.area()+c1.area()-circle_intersection_area(c0,c1);
  if (!a) {
    CHERROR<<"Circled area == 0"<<ENDL;
    exit(1);
  } else return polygonArea(nodes)/a;
};
*/
