/*
  Copyright (C) 2009 Facundo Domínguez 

  This file is part of Spacejunk.

  Spacejunk is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Foobar is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "testparsercomb.h"
#include "parsercombinators.h"
#include <sstream>
#include <iostream>
#include <assert.h>

using namespace std;

void TestParserCombinators(){
  istringstream s;
  CharTokenizer * ct=new StreamTokenizer(s);
  Tokenizer *t=new ConfigTokenizer(ct);
  StreamTokenizer * st;
  Parser p=t;
  delete t;
  delete ct;

  s.clear();
  s.str("<c><d><e>pp</e></d><d></d></c>");
  ct=new StreamTokenizer(s);
  t=new ConfigTokenizer(ct);
  p=t;
  int count=0;
  PARSEbegin(Parser,p)
   S("<c>");
   MANYbegin
     TRYbegin;
       S("<d>");
       MANY(TRY(S("<e>"))S("pp")S("</e>"))
       S("</d>");
     TRYend;
     count++;
   MANYend;
   S("</c>");
  PARSEend;
  if (p.error()) cout<<"Error: "<<p.errorMessage()<<endl;
  assert(count==2);
  assert(!p.error());
  delete t;
  delete ct;

  s.clear();
  s.str("<c>pp</c>");
  ct=new StreamTokenizer(s);
  t=new ConfigTokenizer(ct);
  p=t;
  PARSEbegin(Parser,p)
   S("<c>")
   OPT( T("p") TRY(S("p"))
     OR T("q") TRY(S("q")) )
   S("</c>")
  PARSEend;
  if (p.error()) cout<<"Error: "<<p.errorMessage()<<endl;
  assert(p.error());
  delete t;
  delete ct;


  s.clear();
  s.str("<d>pp</d></f>");
  ct=new StreamTokenizer(s);
  t=new ConfigTokenizer(ct);
  p=t;
  PARSE(Parser,p,
   OPT( MANY( TRY(S("<d>"))S("pp")S("</d>") )
     OR S("</d>") )
   S("</c>")
  )
  assert(p.error());
  delete t;
  delete ct;

  s.clear();
  s.str("<d>pp</d></c>");
  ct=new StreamTokenizer(s);
  t=new ConfigTokenizer(ct);
  p=t;
  PARSE(Parser,p,
   OPT(
      MANY( TRY(S("<d>"))S("pp")S("</d>") )
    OR
      S("</d>")
   )
   S("</c>")
  )
  assert(!p.error());
  delete t;
  delete ct;

  s.clear();
  s.str("</d></c>");
  ct=new StreamTokenizer(s);
  t=new ConfigTokenizer(ct);
  p=t;
  PARSEbegin(Parser,p);
   OPT( TRY( S("<d>")S("pp")S("</d>") ) OR S("</d>") )
   S("</c>");
  PARSEend;
  if (p.error()) cout<<"Error: "<<p.errorMessage()<<endl;
  assert(!p.error());
  delete t;
  delete ct;

  string str;
  s.clear();
  s.str("<d>pp</d><d>pp</d><d>qq</d>");
  st=new StreamTokenizer(s);
  Lexer l=st;
  count=0;
  basic_string<wchar_t> wstr;
  PARSEbegin(Lexer,l)
    MANYbegin
      S("<d>");
      ID(&wstr);
      if (wstr==L"pp") count++;
      S("</d>");
    MANYend
    PEOF
  PARSEend;
  if (l.error()) cout<<"Error: "<<l.errorMessage()<<endl;
  assert(!l.error());
  assert(wstr==L"qq");
  assert(count==2);
  delete st;

  s.clear();
  s.str("1234.45 123 .4 6.");
  st=new StreamTokenizer(s);
  l=st;
  double d;
  count=0;
  PARSEbegin(Lexer,l)
    LFLOAT(&d);
    assert(d==1234.45);
    LFLOAT(&d);
    assert(d==123);
    OPTbegin;
      LFLOAT(&d);assert(false);
      OR;
      MANY(NOTONEOF(" "));
      WS;
    OPTend;
    OPTbegin;
      TRY(LFLOAT(&d));
      assert(false);
      OR;
      MANY(NOTONEOF(" "));
      WS;
    OPTend;
    PEOF
  PARSEend;
  if (l.error()) cout<<"Error: "<<l.errorMessage()<<endl;
  assert(!l.error());
  delete st;

  s.clear();
  s.str("");
  st=new StreamTokenizer(s);
  l=st;
  PARSEbegin(Lexer,l)
  PARSEend;
  if (l.error()) cout<<"Error: "<<l.errorMessage()<<endl;
  assert(!l.error());
  delete st;

};
